@@ -35,8 +35,6 @@ import sys
 from ast import literal_eval
 from collections import defaultdict
 
-from pynicotine.logfacility import log
-
 
 class Config:
     """
@@ -55,10 +53,88 @@ class Config:
     def __init__(self):
 
         config_dir, self.data_dir = self.get_user_directories()
-        self.filename = os.path.join(config_dir, 'config')
-        self.plugin_dir = os.path.join(self.data_dir, 'plugins')
+        self.filename = os.path.join(config_dir, "config")
+        self.plugin_dir = os.path.join(self.data_dir, "plugins")
+        self.version = "3.1.0"
 
-        self.parser = configparser.RawConfigParser(strict=False)
+        self.parser = configparser.ConfigParser(strict=False, interpolation=None)
+        self.sections = defaultdict(dict)
+        self.defaults = {}
+
+    @staticmethod
+    def get_user_directories():
+        """ Returns a tuple:
+        - the config directory
+        - the data directory """
+
+        if sys.platform == "win32":
+            try:
+                data_dir = os.path.join(os.environ['APPDATA'], 'nicotine')
+            except KeyError:
+                data_dir, _filename = os.path.split(sys.argv[0])
+
+            config_dir = os.path.join(data_dir, "config")
+            return config_dir, data_dir
+
+        home = os.path.expanduser("~")
+
+        legacy_dir = os.path.join(home, '.nicotine')
+
+        if os.path.isdir(legacy_dir):
+            return legacy_dir, legacy_dir
+
+        def xdg_path(xdg, default):
+            path = os.environ.get(xdg)
+
+            path = path.split(':')[0] if path else default
+
+            return os.path.join(path, 'nicotine')
+
+        config_dir = xdg_path('XDG_CONFIG_HOME', os.path.join(home, '.config'))
+        data_dir = xdg_path('XDG_DATA_HOME', os.path.join(home, '.local', 'share'))
+
+        return config_dir, data_dir
+
+    def create_config_folder(self):
+        """ Create the folder for storing the config file in, if the folder
+        doesn't exist """
+
+        path, _filename = os.path.split(self.filename)
+
+        if not path:
+            # Only file name specified, use current folder
+            return True
+
+        try:
+            if not os.path.isdir(path):
+                os.makedirs(path)
+
+        except OSError as msg:
+            from pynicotine.logfacility import log
+
+            log.add(_("Can't create directory '%(path)s', reported error: %(error)s"),
+                    {'path': path, 'error': msg})
+            return False
+
+        return True
+
+    def create_data_folder(self):
+        """ Create the folder for storing data in (aliases, shared files etc.),
+        if the folder doesn't exist """
+
+        try:
+            if not os.path.isdir(self.data_dir):
+                os.makedirs(self.data_dir)
+
+        except OSError as msg:
+            from pynicotine.logfacility import log
+
+            log.add(_("Can't create directory '%(path)s', reported error: %(error)s"),
+                    {'path': self.data_dir, 'error': msg})
+
+    def load_config(self):
+
+        from pynicotine.utils import load_file
 
         log_dir = os.path.join(self.data_dir, "logs")
         self.defaults = {
@@ -66,6 +142,7 @@ class Config:
                 "server": ('server.slsknet.org', 2242),
                 "login": '',
                 "passw": '',
+                "interface": '',
                 "ctcpmsgs": False,
                 "autosearch": [],
                 "autoreply": "",
@@ -89,13 +166,17 @@ class Config:
                 "incompletedir": os.path.join(self.data_dir, 'incomplete'),
                 "downloaddir": os.path.join(self.data_dir, 'downloads'),
                 "uploaddir": os.path.join(self.data_dir, 'received'),
+                "usernamesubfolders": False,
                 "sharedownloaddir": False,
                 "shared": [],
                 "buddyshared": [],
                 "uploadbandwidth": 10,
                 "uselimit": False,
-                "uploadlimit": 150,
+                "usealtlimits": False,
+                "uploadlimit": 1000,
+                "uploadlimitalt": 100,
                 "downloadlimit": 0,
+                "downloadlimitalt": 100,
                 "preferfriends": False,
                 "useupslots": False,
                 "uploadslots": 2,
@@ -112,11 +193,12 @@ class Config:
                 "queuelimit": 10000,
                 "filelimit": 100,
                 "friendsonly": False,
+                "buddysharestrustedonly": False,
                 "friendsnolimits": False,
                 "enablebuddyshares": False,
                 "enabletransferbuttons": True,
-                "groupdownloads": 1,
-                "groupuploads": 1,
+                "groupdownloads": "folder_grouping",
+                "groupuploads": "folder_grouping",
                 "geoblock": False,
                 "geopanic": False,
                 "geoblockcc": [""],
@@ -126,7 +208,7 @@ class Config:
                 "autoclear_uploads": False,
                 "uploadsinsubdirs": True,
                 "rescanonstartup": True,
-                "enablefilters": True,
+                "enablefilters": False,
                 "downloadregexp": "",
                 "downloadfilters": [
                     ["desktop.ini", 1],
@@ -174,7 +256,7 @@ class Config:
 
             "logging": {
                 "debug": False,
-                "debugmodes": [0, 1],
+                "debugmodes": [],
                 "debuglogsdir": os.path.join(log_dir, "debug"),
                 "logcollapsed": True,
                 "transferslogsdir": os.path.join(log_dir, "transfers"),
@@ -211,7 +293,7 @@ class Config:
 
             "searches": {
                 "expand_searches": True,
-                "group_searches": 1,
+                "group_searches": "folder_grouping",
                 "maxresults": 50,
                 "re_filter": False,
                 "history": [],
@@ -225,8 +307,7 @@ class Config:
                 "filterbr": [],
                 "filtertype": [],
                 "search_results": True,
-                "max_displayed_results": 1000,
-                "max_stored_results": 1500,
+                "max_displayed_results": 1500,
                 "min_search_chars": 3,
                 "remove_special_chars": True
             },
@@ -256,7 +337,6 @@ class Config:
                 "tab_hilite": "#497ec2",
                 "tab_changed": "#497ec2",
                 "tab_select_previous": True,
-                "tab_reorderable": True,
                 "tabmain": "Top",
                 "tabrooms": "Top",
                 "tabprivate": "Top",
@@ -315,7 +395,11 @@ class Config:
                 "xposition": -1,
                 "yposition": -1,
                 "maximized": True,
-                "urgencyhint": True
+                "urgencyhint": True,
+                "private_search_results": False,
+                "private_shares": False,
+                "file_path_tooltips": True,
+                "reverse_file_paths": True
             },
 
             "private_rooms": {
@@ -373,81 +457,13 @@ class Config:
             self.defaults['players']['npplayer'] = 'other'
 
         # Initialize config with default values
-        self.sections = defaultdict(dict)
-
         for key, value in self.defaults.items():
             self.sections[key] = value.copy()
 
-    def get_user_directories(self):
-        """ Returns a tuple:
-        - the config directory
-        - the data directory """
-
-        if sys.platform == "win32":
-            try:
-                data_dir = os.path.join(os.environ['APPDATA'], 'nicotine')
-            except KeyError:
-                data_dir, x = os.path.split(sys.argv[0])
-
-            config_dir = os.path.join(data_dir, "config")
-            return config_dir, data_dir
-
-        home = os.path.expanduser("~")
-
-        legacy_dir = os.path.join(home, '.nicotine')
-
-        if os.path.isdir(legacy_dir):
-            return legacy_dir, legacy_dir
-
-        def xdg_path(xdg, default):
-            path = os.environ.get(xdg)
-
-            path = path.split(':')[0] if path else default
-
-            return os.path.join(path, 'nicotine')
-
-        config_dir = xdg_path('XDG_CONFIG_HOME', os.path.join(home, '.config'))
-        data_dir = xdg_path('XDG_DATA_HOME', os.path.join(home, '.local', 'share'))
-
-        return config_dir, data_dir
-
-    def create_config_folder(self):
-        """ Create the folder for storing the config file in, if the folder
-        doesn't exist """
-
-        path, fn = os.path.split(self.filename)
-        try:
-            if not os.path.isdir(path):
-                os.makedirs(path)
-
-        except OSError as msg:
-            log.add_warning(_("Can't create directory '%(path)s', reported error: %(error)s"), {'path': path, 'error': msg})
-            return False
-
-        return True
-
-    def create_data_folder(self):
-        """ Create the folder for storing data in (aliases, shared files etc.),
-        if the folder doesn't exist """
-
-        try:
-            if not os.path.isdir(self.data_dir):
-                os.makedirs(self.data_dir)
-
-        except OSError as msg:
-            log.add_warning(_("Can't create directory '%(path)s', reported error: %(error)s"), {'path': self.data_dir, 'error': msg})
-
-    def load_config(self):
-
         self.create_config_folder()
         self.create_data_folder()
 
-        try:
-            self.parse_config()
-
-        except UnicodeDecodeError:
-            self.convert_config()
-            self.parse_config()
+        load_file(self.filename, self.parse_config)
 
         # Clean up old config options
         self.remove_old_options()
@@ -458,28 +474,25 @@ class Config:
         # Load command aliases from legacy file
         try:
             if not self.sections["server"]["command_aliases"] and os.path.exists(self.filename + ".alias"):
-                with open(self.filename + ".alias", 'rb') as f:
+                with open(self.filename + ".alias", 'rb') as file_handle:
                     from pynicotine.utils import RestrictedUnpickler
-                    self.sections["server"]["command_aliases"] = RestrictedUnpickler(f, encoding='utf-8').load()
+                    self.sections["server"]["command_aliases"] = RestrictedUnpickler(
+                        file_handle, encoding='utf-8').load()
 
         except Exception:
             return
 
-    def parse_config(self):
+    def parse_config(self, filename):
         """ Parses the config file """
 
         try:
-            with open(self.filename, 'a+', encoding="utf-8") as f:
-                f.seek(0)
-                self.parser.read_file(f)
+            with open(filename, 'a+', encoding="utf-8") as file_handle:
+                file_handle.seek(0)
+                self.parser.read_file(file_handle)
 
-        except configparser.ParsingError:
-            # Ignore parsing errors, the offending lines are removed later
-            pass
-
-        except Exception as e:
-            # Miscellaneous failure, default config will be used
-            log.add(_("Unable to parse config file: %s"), e)
+        except UnicodeDecodeError:
+            self.convert_config()
+            self.parse_config(filename)
 
     def convert_config(self):
         """ Converts the config to utf-8.
@@ -494,25 +507,22 @@ class Config:
 
         os.rename(self.filename, self.filename + ".conv")
 
-        with open(self.filename + ".conv", 'rb') as f:
-            rawdata = f.read()
+        with open(self.filename + ".conv", 'rb') as file_handle:
+            rawdata = file_handle.read()
 
         from_encoding = detect(rawdata)['encoding']
 
-        with open(self.filename + ".conv", 'r', encoding=from_encoding) as fr:
-            with open(self.filename, 'w', encoding="utf-8") as fw:
-                for line in fr:
-                    fw.write(line[:-1] + '\r\n')
+        with open(self.filename + ".conv", 'r', encoding=from_encoding) as file_read:
+            with open(self.filename, 'w', encoding="utf-8") as file_write:
+                for line in file_read:
+                    file_write.write(line[:-1] + '\r\n')
 
         os.remove(self.filename + ".conv")
 
     def need_config(self):
 
         # Check if we have specified a username or password
-        if not self.sections["server"]["login"] or \
-                not self.sections["server"]["passw"]:
-
-            log.add(_("You need to specify a username and password before connecting..."))
+        if not self.sections["server"]["login"] or not self.sections["server"]["passw"]:
             return True
 
         return False
@@ -520,16 +530,19 @@ class Config:
     def set_config(self):
         """ Set config values parsed from file earlier """
 
+        from pynicotine.logfacility import log
+
         for i in self.parser.sections():
             for j, val in self.parser.items(i, raw=True):
 
                 # Check if config section exists in defaults
                 if i not in self.defaults:
-                    log.add_warning(_("Unknown config section '%s'"), i)
+                    log.add(_("Unknown config section '%s'"), i)
 
                 # Check if config option exists in defaults
                 elif j not in self.defaults[i] and i != "plugins" and j != "filter":
-                    log.add_warning(_("Unknown config option '%(option)s' in section '%(section)s'"), {'option': j, 'section': i})
+                    log.add(_("Unknown config option '%(option)s' in section '%(section)s'"),
+                            {'option': j, 'section': i})
 
                 else:
                     """ Attempt to get the default value for a config option. If there's no default
@@ -560,9 +573,13 @@ class Config:
                             eval_val = val
 
                         if i != "plugins" and j != "filter":
-                            if (isinstance(default_val, bool) and isinstance(eval_val, int) and eval_val != 0 and eval_val != 1) or \
-                                    (not isinstance(default_val, bool) and type(eval_val) != type(default_val)):
+                            if (isinstance(eval_val, type(default_val))
+                                    or (isinstance(default_val, bool)
+                                        and isinstance(eval_val, int) and eval_val in (0, 1))):
+                                # Value is valid
+                                pass
 
+                            else:
                                 raise TypeError("Invalid config value type detected")
 
                         self.sections[i][j] = eval_val
@@ -571,25 +588,24 @@ class Config:
                         # Value was unexpected, reset option
                         self.sections[i][j] = default_val
 
-                        log.add_warning("Config error: Couldn't decode '%s' section '%s' value '%s', value has been reset", (
+                        log.add("Config error: Couldn't decode '%s' section '%s' value '%s', value has been reset", (
                             (i[:120] + '..') if len(i) > 120 else i,
                             (j[:120] + '..') if len(j) > 120 else j,
                             (val[:120] + '..') if len(val) > 120 else val
-                        )
-                        )
+                        ))
 
         server = self.sections["server"]
 
         # Check if server value is valid
-        if len(server["server"]) != 2 or \
-                not isinstance(server["server"][0], str) or \
-                not isinstance(server["server"][1], int):
+        if (len(server["server"]) != 2
+                or not isinstance(server["server"][0], str)
+                or not isinstance(server["server"][1], int)):
 
             server["server"] = self.defaults["server"]["server"]
 
         # Check if port range value is valid
-        if len(server["portrange"]) != 2 or \
-                not all(isinstance(i, int) for i in server["portrange"]):
+        if (len(server["portrange"]) != 2
+                or not all(isinstance(i, int) for i in server["portrange"])):
 
             server["portrange"] = self.defaults["server"]["portrange"]
 
@@ -691,6 +707,12 @@ class Config:
         # Remove option to toggle chatroom arrow buttons, since they no longer exist
         self.remove_old_option("ui", "chat_hidebuttons")
 
+        # Remove max stored search results, only visible search result limit is used now
+        self.remove_old_option("searches", "max_stored_results")
+
+        # Remove option to specify if tabs are reorderable, since there's no clear advantage
+        self.remove_old_option("ui", "tab_reorderable")
+
     def remove_old_option(self, section, option):
 
         if section in self.parser.sections() and option in self.parser.options(section):
@@ -701,8 +723,8 @@ class Config:
         if section in self.parser.sections():
             self.parser.remove_section(section)
 
-    def write_config_callback(self, f):
-        self.parser.write(f)
+    def write_config_callback(self, filename):
+        self.parser.write(filename)
 
     def write_configuration(self):
 
@@ -711,7 +733,7 @@ class Config:
                 self.parser.add_section(i)
 
             for j in self.sections[i]:
-                self.parser.set(i, j, self.sections[i][j])
+                self.parser.set(i, j, str(self.sections[i][j]))
 
         if not self.create_config_folder():
             return
@@ -726,7 +748,7 @@ class Config:
 
         try:
             if os.path.exists(filename):
-                raise FileExistsError("File %s exists", filename)
+                raise FileExistsError("File %s exists" % filename)
 
             import tarfile
             with tarfile.open(filename, "w:bz2") as tar:
@@ -735,9 +757,9 @@ class Config:
 
                 tar.add(self.filename)
 
-        except Exception as e:
-            print(e)
-            return (True, "Cannot write backup archive: %s" % e)
+        except Exception as error:
+            print(error)
+            return (True, "Cannot write backup archive: %s" % error)
 
         return (False, filename)
 
