@@ -27,24 +27,28 @@ import os
 from sys import maxsize
 from time import time
 
-from gi.repository import Gdk
+from gi.repository import Gio
+from gi.repository import GLib
 from gi.repository import GObject
 from gi.repository import Gtk
 
 from pynicotine.config import config
 from pynicotine.gtkgui.fileproperties import FileProperties
+from pynicotine.gtkgui.utils import connect_key_press_event
 from pynicotine.gtkgui.utils import copy_file_url
-from pynicotine.gtkgui.utils import keyval_to_hardware_keycode
+from pynicotine.gtkgui.utils import copy_text
+from pynicotine.gtkgui.utils import get_key_press_event_args
 from pynicotine.gtkgui.utils import load_ui_elements
-from pynicotine.gtkgui.utils import triggers_context_menu
+from pynicotine.gtkgui.utils import parse_accelerator
 from pynicotine.gtkgui.widgets.popupmenu import PopupMenu
 from pynicotine.gtkgui.widgets.theme import update_widget_visuals
 from pynicotine.gtkgui.widgets.treeview import collapse_treeview
+from pynicotine.gtkgui.widgets.treeview import create_grouping_menu
 from pynicotine.gtkgui.widgets.treeview import initialise_columns
 from pynicotine.gtkgui.widgets.treeview import save_columns
 from pynicotine.gtkgui.widgets.treeview import select_user_row_iter
-from pynicotine.gtkgui.widgets.treeview import set_treeview_selected_row
 from pynicotine.gtkgui.widgets.treeview import show_file_path_tooltip
+from pynicotine.gtkgui.widgets.treeview import verify_grouping_mode
 from pynicotine.transfers import Transfer
 from pynicotine.utils import human_size
 from pynicotine.utils import human_speed
@@ -59,10 +63,23 @@ class TransferList:
 
         load_ui_elements(self, os.path.join(frame.gui_dir, "ui", type + "s.ui"))
         getattr(frame, type + "svbox").add(self.Main)
-        self.widget = widget = getattr(self, type.title() + "List")
+
+        grouping_button = getattr(frame, "ToggleTree%ss" % self.type.title())
+
+        if Gtk.get_major_version() == 4:
+            grouping_button.set_icon_name("view-list-symbolic")
+
+            self.ClearTransfers.set_has_frame(False)
+            self.ClearTransfers.set_label(self.ClearTransfersLabel.get_first_child().get_text())
+        else:
+            grouping_button.set_image(Gtk.Image.new_from_icon_name("view-list-symbolic", Gtk.IconSize.BUTTON))
+
+            self.ClearTransfers.add(self.ClearTransfersLabel)
+
+        self.key_controller = connect_key_press_event(self.Transfers, self.on_key_press_event)
 
         self.last_ui_update = self.last_save = 0
-        self.list = []
+        self.transfer_list = []
         self.users = {}
         self.paths = {}
 
@@ -75,12 +92,13 @@ class TransferList:
         self.statuses["Cannot connect"] = _("Cannot connect")
         self.statuses["User logged off"] = _("User logged off")
         self.statuses["Connection closed by peer"] = _("Connection closed by peer")
+        self.statuses["Disallowed extension"] = _("Disallowed extension")  # Sent by Soulseek NS for filtered extensions
         self.statuses["Aborted"] = _("Aborted")
         self.statuses["Finished"] = _("Finished")
         self.statuses["Filtered"] = _("Filtered")
         self.statuses["File not shared"] = _("File not shared")
         self.statuses["File not shared."] = _("File not shared")  # The official client sends a variant containing a dot
-        self.statuses["Download directory error"] = _("Download directory error")
+        self.statuses["Download folder error"] = _("Download folder error")
         self.statuses["Local file error"] = _("Local file error")
         self.statuses["Remote file error"] = _("Remote file error")
 
@@ -112,8 +130,7 @@ class TransferList:
 
         self.column_numbers = list(range(self.transfersmodel.get_n_columns()))
         self.cols = cols = initialise_columns(
-            type,
-            widget,
+            type, self.Transfers,
             ["user", _("User"), 200, "text", None],
             ["path", _("Path"), 400, "text", None],
             ["filename", _("Filename"), 400, "text", None],
@@ -137,33 +154,41 @@ class TransferList:
         cols["time_elapsed"].set_sort_column_id(8)
         cols["time_left"].set_sort_column_id(9)
 
-        widget.set_model(self.transfersmodel)
+        self.Transfers.set_model(self.transfersmodel)
 
-        self.group_dropdown = getattr(frame, "ToggleTree%ss" % self.type.title())
         self.expand_button = getattr(frame, "Expand%ss" % self.type.title())
 
-        self.group_dropdown.connect("changed", self.on_toggle_tree)
-        self.group_dropdown.set_active(config.sections["transfers"]["group%ss" % self.type])
+        state = GLib.Variant.new_string(verify_grouping_mode(config.sections["transfers"]["group%ss" % self.type]))
+        action = Gio.SimpleAction.new_stateful("%sgrouping" % self.type, GLib.VariantType.new("s"), state)
+        action.connect("change-state", self.on_toggle_tree)
+        frame.MainWindow.add_action(action)
+        action.change_state(state)
+
+        menu = create_grouping_menu(
+            frame.MainWindow, config.sections["transfers"]["group%ss" % self.type], self.on_toggle_tree)
+        grouping_button.set_menu_model(menu)
 
         self.expand_button.connect("toggled", self.on_expand_tree)
         self.expand_button.set_active(config.sections["transfers"]["%ssexpanded" % self.type])
 
         self.popup_menu_users = PopupMenu(frame)
         self.popup_menu_clear = PopupMenu(frame)
+        self.ClearTransfers.set_menu_model(self.popup_menu_clear)
 
-        self.popup_menu = PopupMenu(frame)
+        self.popup_menu = PopupMenu(frame, self.Transfers, self.on_popup_menu)
         self.popup_menu.setup(
             ("#" + "selected_files", None),
             ("", None),
             ("#" + _("Send to _Player"), self.on_play_files),
-            ("#" + _("_Open Folder"), self.on_open_directory),
-            ("#" + _("File P_roperties"), self.on_file_properties),
+            ("#" + _("_Open in File Manager"), self.on_open_directory),
             ("", None),
             ("#" + _("Copy _File Path"), self.on_copy_file_path),
             ("#" + _("Copy _URL"), self.on_copy_url),
             ("#" + _("Copy Folder URL"), self.on_copy_dir_url),
             ("", None),
             ("#" + _("_Search"), self.on_file_search),
+            ("#" + _("_Browse Folder(s)"), self.on_browse_folder),
+            ("#" + _("File P_roperties"), self.on_file_properties),
             (">" + _("User(s)"), self.popup_menu_users),
             ("", None),
             ("#" + _("_Retry"), self.on_retry_transfer),
@@ -175,40 +200,36 @@ class TransferList:
 
         self.update_visuals()
 
-    def init_interface(self, list):
+    def server_login(self):
 
-        self.list = list
-        self.widget.set_sensitive(True)
+        self.transfer_list = getattr(self.frame.np.transfers, "%ss" % self.type)
+        self.Transfers.set_sensitive(True)
         self.update()
 
+    def server_disconnect(self):
+
+        self.Transfers.set_sensitive(False)
+        self.clear()
+        self.transfer_list = []
+
     def rebuild_transfers(self):
-
-        if self.frame.np.transfers is None:
-            return
-
         self.clear()
         self.update()
 
     def save_columns(self):
-        save_columns(self.type, self.widget.get_columns())
+        save_columns(self.type, self.Transfers.get_columns())
 
     def update_visuals(self):
 
         for widget in list(self.__dict__.values()):
             update_widget_visuals(widget, list_font_target="transfersfont")
 
-    def conn_close(self):
-
-        self.widget.set_sensitive(False)
-        self.list = []
-        self.clear()
-
     def select_transfers(self):
 
         self.selected_transfers = set()
         self.selected_users = set()
 
-        model, paths = self.widget.get_selection().get_selected_rows()
+        model, paths = self.Transfers.get_selection().get_selected_rows()
 
         for path in paths:
             iterator = model.get_iter(path)
@@ -267,7 +288,7 @@ class TransferList:
 
     def update(self, transfer=None, forceupdate=False):
 
-        if not self.widget.get_sensitive():
+        if not self.Transfers.get_sensitive():
             """ List is not initialized """
             return
 
@@ -275,17 +296,17 @@ class TransferList:
 
         if (curtime - self.last_save) > 15:
 
-            """ Save downloads list to file every 15 seconds """
+            """ Save list of transfers to file every 15 seconds """
 
             if self.frame.np.transfers is not None:
-                self.frame.np.transfers.save_downloads()
+                self.frame.np.transfers.save_transfers("downloads")
+                self.frame.np.transfers.save_transfers("uploads")
 
             self.last_save = curtime
 
         finished = (transfer is not None and transfer.status == "Finished")
 
-        if forceupdate or finished or \
-                (curtime - self.last_ui_update) > 1:
+        if forceupdate or finished or (curtime - self.last_ui_update) > 1:
             self.frame.update_bandwidth()
 
         if not forceupdate and self.frame.current_tab_label != self.tab_label:
@@ -295,12 +316,11 @@ class TransferList:
         if transfer is not None:
             self.update_specific(transfer)
 
-        elif self.list is not None:
-            for transfer in reversed(self.list):
+        elif self.transfer_list:
+            for transfer in reversed(self.transfer_list):
                 self.update_specific(transfer)
 
-        if forceupdate or finished or \
-                (curtime - self.last_ui_update) > 1:
+        if forceupdate or finished or (curtime - self.last_ui_update) > 1:
 
             """ Unless a transfer finishes, use a cooldown to avoid updating
             too often """
@@ -465,8 +485,8 @@ class TransferList:
             percent = 100
 
         # Modify old transfer
-        if transfer.iter is not None:
-            initer = transfer.iter
+        if transfer.iterator is not None:
+            initer = transfer.iterator
 
             self.transfersmodel.set_value(initer, 3, hstatus)
             self.transfersmodel.set_value(initer, 4, hplace)
@@ -481,27 +501,66 @@ class TransferList:
             self.transfersmodel.set_value(initer, 14, GObject.Value(GObject.TYPE_UINT64, speed))
             self.transfersmodel.set_value(initer, 15, GObject.Value(GObject.TYPE_UINT64, elapsed))
             self.transfersmodel.set_value(initer, 17, GObject.Value(GObject.TYPE_UINT64, place))
+            return
 
-        else:
-            fn = transfer.filename
-            user = transfer.user
-            shortfn = fn.split("\\")[-1]
-            filecount = 1
+        fn = transfer.filename
+        user = transfer.user
+        shortfn = fn.split("\\")[-1]
+        filecount = 1
 
-            if self.tree_users != "ungrouped":
-                # Group by folder or user
+        if self.tree_users != "ungrouped":
+            # Group by folder or user
 
-                empty_int = 0
-                empty_str = ""
+            empty_int = 0
+            empty_str = ""
 
-                if user not in self.users:
-                    # Create Parent if it doesn't exist
-                    # ProgressRender not visible (last column sets 4th column)
-                    self.users[user] = self.transfersmodel.insert_with_values(
-                        None, -1, self.column_numbers,
+            if user not in self.users:
+                # Create Parent if it doesn't exist
+                # ProgressRender not visible (last column sets 4th column)
+                self.users[user] = self.transfersmodel.insert_with_values(
+                    None, -1, self.column_numbers,
+                    [
+                        user,
+                        empty_str,
+                        empty_str,
+                        empty_str,
+                        empty_str,
+                        empty_int,
+                        empty_str,
+                        empty_str,
+                        empty_str,
+                        empty_str,
+                        empty_str,
+                        empty_str,
+                        empty_int,
+                        empty_int,
+                        empty_int,
+                        empty_int,
+                        filecount,
+                        empty_int,
+                        lambda: None
+                    ]
+                )
+
+            parent = self.users[user]
+
+            if self.tree_users == "folder_grouping":
+                # Group by folder
+
+                """ Paths can be empty if files are downloaded individually, make sure we
+                don't add files to the wrong user in the TreeView """
+                path = transfer.path
+                user_path = user + path
+
+                if config.sections["ui"]["reverse_file_paths"]:
+                    path = '/'.join(reversed(path.split('/')))
+
+                if user_path not in self.paths:
+                    self.paths[user_path] = self.transfersmodel.insert_with_values(
+                        self.users[user], -1, self.column_numbers,
                         [
                             user,
-                            empty_str,
+                            path,
                             empty_str,
                             empty_str,
                             empty_str,
@@ -522,94 +581,60 @@ class TransferList:
                         ]
                     )
 
-                parent = self.users[user]
+                parent = self.paths[user_path]
+        else:
+            # No grouping
+            # We use this list to get the total number of users
+            self.users.setdefault(user, set()).add(transfer)
+            parent = None
 
-                if self.tree_users == "folder_grouping":
-                    # Group by folder
+        # Add a new transfer
+        if self.tree_users == "folder_grouping":
+            # Group by folder, path not visible
+            path = ""
+        else:
+            path = transfer.path
 
-                    """ Paths can be empty if files are downloaded individually, make sure we
-                    don't add files to the wrong user in the TreeView """
-                    path = transfer.path
-                    user_path = user + path
-                    reverse_path = '/'.join(reversed(path.split('/')))
+            if config.sections["ui"]["reverse_file_paths"]:
+                path = '/'.join(reversed(path.split('/')))
 
-                    if user_path not in self.paths:
-                        self.paths[user_path] = self.transfersmodel.insert_with_values(
-                            self.users[user], -1, self.column_numbers,
-                            [
-                                user,
-                                reverse_path,
-                                empty_str,
-                                empty_str,
-                                empty_str,
-                                empty_int,
-                                empty_str,
-                                empty_str,
-                                empty_str,
-                                empty_str,
-                                empty_str,
-                                empty_str,
-                                empty_int,
-                                empty_int,
-                                empty_int,
-                                empty_int,
-                                filecount,
-                                empty_int,
-                                lambda: None
-                            ]
-                        )
-
-                    parent = self.paths[user_path]
-            else:
-                # No grouping
-                # We use this list to get the total number of users
-                self.users.setdefault(user, set()).add(transfer)
-                parent = None
-
-            # Add a new transfer
-            if self.tree_users == "folder_grouping":
-                # Group by folder, path not visible
-                path = ""
-            else:
-                path = '/'.join(reversed(transfer.path.split('/')))
-
-            iterator = self.transfersmodel.insert_with_values(
-                parent, -1, self.column_numbers,
-                (
-                    user,
-                    path,
-                    shortfn,
-                    hstatus,
-                    hplace,
-                    GObject.Value(GObject.TYPE_UINT64, percent),
-                    hsize,
-                    hspeed,
-                    helapsed,
-                    left,
-                    fn,
-                    status,
-                    GObject.Value(GObject.TYPE_UINT64, size),
-                    GObject.Value(GObject.TYPE_UINT64, icurrentbytes),
-                    GObject.Value(GObject.TYPE_UINT64, speed),
-                    GObject.Value(GObject.TYPE_UINT64, elapsed),
-                    GObject.Value(GObject.TYPE_UINT64, filecount),
-                    GObject.Value(GObject.TYPE_UINT64, place),
-                    transfer
-                )
+        iterator = self.transfersmodel.insert_with_values(
+            parent, -1, self.column_numbers,
+            (
+                user,
+                path,
+                shortfn,
+                hstatus,
+                hplace,
+                GObject.Value(GObject.TYPE_UINT64, percent),
+                hsize,
+                hspeed,
+                helapsed,
+                left,
+                fn,
+                status,
+                GObject.Value(GObject.TYPE_UINT64, size),
+                GObject.Value(GObject.TYPE_UINT64, icurrentbytes),
+                GObject.Value(GObject.TYPE_UINT64, speed),
+                GObject.Value(GObject.TYPE_UINT64, elapsed),
+                GObject.Value(GObject.TYPE_UINT64, filecount),
+                GObject.Value(GObject.TYPE_UINT64, place),
+                transfer
             )
-            transfer.iter = iterator
+        )
+        transfer.iterator = iterator
 
-            # Expand path
-            if parent is not None:
-                transfer_path = self.transfersmodel.get_path(iterator)
+        # Expand path
+        if parent is not None:
+            transfer_path = self.transfersmodel.get_path(iterator)
 
-                if self.tree_users == "folder_grouping":
-                    # Group by folder, we need the user path to expand it
-                    user_path = self.transfersmodel.get_path(self.users[user])
-                else:
-                    user_path = None
+            if self.tree_users == "folder_grouping":
+                # Group by folder, we need the user path to expand it
+                user_path = self.transfersmodel.get_path(self.users[user])
+            else:
+                user_path = None
 
-                self.expand(transfer_path, user_path)
+            self.expand(transfer_path, user_path)
 
     def retry_transfers(self):
         for transfer in self.selected_transfers:
@@ -640,16 +665,16 @@ class TransferList:
             del self.frame.np.transfers.transfer_request_times[transfer]
 
         if not cleartreeviewonly:
-            self.list.remove(transfer)
+            self.transfer_list.remove(transfer)
 
-        if transfer.iter is not None:
-            self.transfersmodel.remove(transfer.iter)
+        if transfer.iterator is not None:
+            self.transfersmodel.remove(transfer.iterator)
 
         self.update_parent_rows(only_remove=True)
 
     def clear_transfers(self, status):
 
-        for transfer in self.list.copy():
+        for transfer in self.transfer_list.copy():
             if transfer.status in status:
                 self.frame.np.transfers.abort_transfer(transfer, send_fail_message=True)
                 self.remove_specific(transfer)
@@ -662,11 +687,121 @@ class TransferList:
         self.selected_users = set()
         self.transfersmodel.clear()
 
-        if self.list is not None:
-            for transfer in self.list:
-                transfer.iter = None
+        for transfer in self.transfer_list:
+            transfer.iterator = None
 
-    def double_click(self, event):
+    def add_popup_menu_user(self, popup, user):
+
+        popup.setup_user_menu(user)
+        popup.setup(
+            ("", None),
+            ("#" + _("Select User's Transfers"), self.on_select_user_transfers, user)
+        )
+
+        popup.toggle_user_items()
+
+    def populate_popup_menu_users(self):
+
+        self.popup_menu_users.clear()
+
+        if not self.selected_users:
+            return
+
+        # Multiple users, create submenus for each user
+        if len(self.selected_users) > 1:
+            for user in self.selected_users:
+                popup = PopupMenu(self.frame)
+                self.add_popup_menu_user(popup, user)
+                self.popup_menu_users.setup(
+                    (">" + user, popup)
+                )
+            return
+
+        # Single user, add items directly to "User(s)" submenu
+        user = next(iter(self.selected_users), None)
+        self.add_popup_menu_user(self.popup_menu_users, user)
+
+    def expand(self, transfer_path, user_path):
+
+        if self.expand_button.get_active():
+            self.Transfers.expand_to_path(transfer_path)
+
+        elif user_path and self.tree_users == "folder_grouping":
+            # Group by folder, show user folders in collapsed mode
+
+            self.Transfers.expand_to_path(user_path)
+
+    def on_expand_tree(self, widget):
+
+        expand_button_icon = getattr(self.frame, "Expand%ssImage" % self.type.title())
+        expanded = widget.get_active()
+
+        if expanded:
+            icon_name = "go-up-symbolic"
+            self.Transfers.expand_all()
+
+        else:
+            icon_name = "go-down-symbolic"
+            collapse_treeview(self.Transfers, self.tree_users)
+
+        expand_button_icon.set_property("icon-name", icon_name)
+
+        config.sections["transfers"]["%ssexpanded" % self.type] = expanded
+        config.write_configuration()
+
+    def on_toggle_tree(self, action, state):
+
+        mode = state.get_string()
+        active = mode != "ungrouped"
+
+        config.sections["transfers"]["group%ss" % self.type] = mode
+        self.Transfers.set_show_expanders(active)
+        self.expand_button.set_visible(active)
+
+        self.tree_users = mode
+        self.rebuild_transfers()
+
+        action.set_state(state)
+
+    def on_tooltip(self, widget, x, y, keyboard_mode, tooltip):
+        return show_file_path_tooltip(widget, x, y, tooltip, 10)
+
+    def on_popup_menu(self, menu, widget):
+
+        self.select_transfers()
+        num_selected_transfers = len(self.selected_transfers)
+
+        actions = menu.get_actions()
+        users = len(self.selected_users) > 0
+        files = num_selected_transfers > 0
+
+        actions[_("User(s)")].set_enabled(users)  # Users Menu
+        self.populate_popup_menu_users()
+
+        if files:
+            act = True
+        else:
+            # Disable options
+            # Send to player, File manager, file properties, Copy File Path, Copy URL, Copy Folder URL, Search filename
+            act = False
+
+        for i in (_("Send to _Player"), _("_Open in File Manager"), _("File P_roperties"),
+                  _("Copy _File Path"), _("Copy _URL"), _("Copy Folder URL"), _("_Search")):
+            actions[i].set_enabled(act)
+
+        if not users or not files:
+            # Disable options
+            # Retry, Abort, Clear
+            act = False
+        else:
+            act = True
+
+        for i in (_("_Retry"), _("Abor_t"), _("_Clear")):
+            actions[i].set_enabled(act)
+
+        menu.set_num_selected_files(num_selected_transfers)
+
+    def on_row_activated(self, treeview, path, column):
 
         self.select_transfers()
         dc = config.sections["transfers"]["%s_doubleclick" % self.type]
@@ -684,115 +819,6 @@ class TransferList:
         elif dc == 6:  # Retry
             self.retry_transfers()
 
-    def populate_popup_menu_users(self):
-
-        self.popup_menu_users.clear()
-
-        if not self.selected_users:
-            return
-
-        for user in self.selected_users:
-            popup = PopupMenu(self.frame)
-            popup.setup_user_menu(user)
-            popup.setup(
-                ("", None),
-                ("#" + _("Select User's Transfers"), self.on_select_user_transfers, user)
-            )
-
-            popup.toggle_user_items()
-            self.popup_menu_users.setup(
-                (">" + user, popup)
-            )
-
-    def expand(self, transfer_path, user_path):
-
-        if self.expand_button.get_active():
-            self.widget.expand_to_path(transfer_path)
-
-        elif user_path and self.tree_users == "folder_grouping":
-            # Group by folder, show user folders in collapsed mode
-
-            self.widget.expand_to_path(user_path)
-
-    def on_expand_tree(self, widget):
-
-        expand_button_icon = getattr(self.frame, "Expand%ssImage" % self.type.title())
-        expanded = widget.get_active()
-
-        if expanded:
-            self.widget.expand_all()
-            expand_button_icon.set_from_icon_name("go-up-symbolic", Gtk.IconSize.BUTTON)
-        else:
-            collapse_treeview(self.widget, self.tree_users)
-            expand_button_icon.set_from_icon_name("go-down-symbolic", Gtk.IconSize.BUTTON)
-
-        config.sections["transfers"]["%ssexpanded" % self.type] = expanded
-        config.write_configuration()
-
-    def on_toggle_tree(self, widget):
-
-        active = widget.get_active()
-
-        config.sections["transfers"]["group%ss" % self.type] = active
-        self.widget.set_show_expanders(active)
-        self.expand_button.set_visible(active)
-
-        self.tree_users = widget.get_active_id()
-        self.rebuild_transfers()
-
-    def on_tooltip(self, widget, x, y, keyboard_mode, tooltip):
-        return show_file_path_tooltip(widget, x, y, tooltip, 10)
-
-    def on_popup_menu(self, *args):
-
-        self.select_transfers()
-        num_selected_transfers = len(self.selected_transfers)
-
-        actions = self.popup_menu.get_actions()
-        users = len(self.selected_users) > 0
-        files = num_selected_transfers > 0
-
-        actions[_("User(s)")].set_enabled(users)  # Users Menu
-        self.populate_popup_menu_users()
-
-        if files:
-            act = True
-        else:
-            # Disable options
-            # Send to player, File manager, file properties, Copy File Path, Copy URL, Copy Folder URL, Search filename
-            act = False
-
-        for i in (_("Send to _Player"), _("_Open Folder"), _("File P_roperties"),
-                  _("Copy _File Path"), _("Copy _URL"), _("Copy Folder URL"), _("_Search")):
-            actions[i].set_enabled(act)
-
-        if not users or not files:
-            # Disable options
-            # Retry, Abort, Clear
-            act = False
-        else:
-            act = True
-
-        for i in (_("_Retry"), _("Abor_t"), _("_Clear")):
-            actions[i].set_enabled(act)
-
-        self.popup_menu.set_num_selected_files(num_selected_transfers)
-
-        self.popup_menu.popup()
-        return True
-
-    def on_list_clicked(self, widget, event):
-
-        if triggers_context_menu(event):
-            set_treeview_selected_row(widget, event)
-            return self.on_popup_menu()
-
-        if event.button == 1 and event.type == Gdk.EventType._2BUTTON_PRESS:
-            self.double_click(event)
-            return True
-
-        return False
-
     def on_select_user_transfers(self, *args):
 
         if not self.selected_users:
@@ -800,8 +826,8 @@ class TransferList:
 
         selected_user = args[-1]
 
-        sel = self.widget.get_selection()
-        fmodel = self.widget.get_model()
+        sel = self.Transfers.get_selection()
+        fmodel = self.Transfers.get_model()
         sel.unselect_all()
 
         iterator = fmodel.get_iter_first()
@@ -810,49 +836,46 @@ class TransferList:
 
         self.select_transfers()
 
-    def on_key_press_event(self, widget, event):
+    def on_key_press_event(self, *args):
 
-        keycode = event.hardware_keycode
+        keyval, keycode, state = get_key_press_event_args(*args)
         self.select_transfers()
 
-        if keycode in keyval_to_hardware_keycode(Gdk.KEY_t):
+        keycodes, mods = parse_accelerator("t")
+
+        if keycode in keycodes:
             self.abort_transfers()
+            return True
 
-        elif keycode in keyval_to_hardware_keycode(Gdk.KEY_r):
+        keycodes, mods = parse_accelerator("r")
+
+        if keycode in keycodes:
             self.retry_transfers()
+            return True
 
-        elif event.get_state() & Gdk.ModifierType.CONTROL_MASK and \
-                keycode in keyval_to_hardware_keycode(Gdk.KEY_c):
+        keycodes, mods = parse_accelerator("<Primary>c")
+
+        if state & mods and keycode in keycodes:
             self.on_copy_file_path()
+            return True
 
-        elif keycode in keyval_to_hardware_keycode(Gdk.KEY_Delete):
+        keycodes, mods = parse_accelerator("Delete")
+
+        if keycode in keycodes:
             self.abort_transfers(clear=True)
+            return True
 
-        else:
-            # No key match, continue event
-            return False
-
-        widget.stop_emission_by_name("key_press_event")
-        return True
+        # No key match, continue event
+        return False
 
     def on_file_properties(self, *args):
 
-        if not self.frame.np.transfers:
-            return
-
         data = []
-        model, paths = self.widget.get_selection().get_selected_rows()
 
-        for path in paths:
-            iterator = model.get_iter(path)
-            transfer = model.get_value(iterator, 18)
-
-            if not isinstance(transfer, Transfer):
-                continue
-
-            user = model.get_value(iterator, 0)
-            filename = model.get_value(iterator, 2)
-            fullname = model.get_value(iterator, 10)
+        for transfer in self.selected_transfers:
+            user = transfer.user
+            fullname = transfer.filename
+            filename = fullname.split("\\")[-1]
             size = speed = length = queue = immediate = num = country = bitratestr = ""
 
             size = str(human_size(transfer.size))
@@ -880,7 +903,7 @@ class TransferList:
                 "country": country
             })
 
-        if paths:
+        if data:
             FileProperties(self.frame, data).show()
 
     def on_copy_file_path(self, *args):
@@ -888,21 +911,21 @@ class TransferList:
         transfer = next(iter(self.selected_transfers), None)
 
         if transfer:
-            self.frame.clipboard.set_text(transfer.filename, -1)
+            copy_text(transfer.filename)
 
     def on_copy_url(self, *args):
 
         transfer = next(iter(self.selected_transfers), None)
 
         if transfer:
-            copy_file_url(transfer.user, transfer.filename, self.frame.clipboard)
+            copy_file_url(transfer.user, transfer.filename)
 
     def on_copy_dir_url(self, *args):
 
         transfer = next(iter(self.selected_transfers), None)
 
         if transfer:
-            copy_file_url(transfer.user, transfer.filename.rsplit('\\', 1)[0] + '\\', self.frame.clipboard)
+            copy_file_url(transfer.user, transfer.filename.rsplit('\\', 1)[0] + '\\')
 
     def on_retry_transfer(self, *args):
         self.select_transfers()
@@ -921,7 +944,11 @@ class TransferList:
         dialog.destroy()
 
         if response_id == Gtk.ResponseType.OK:
-            self.clear_transfers(["Queued"])
+            if data == "queued":
+                self.clear_transfers(["Queued"])
+
+            elif data == "all":
+                self.clear()
 
     def on_clear_queued(self, *args):
         self.clear_transfers(["Queued"])
