@@ -24,21 +24,17 @@
 import os
 import time
 
-from gi.repository import Gdk
 from gi.repository import GLib
 from gi.repository import GObject
 from gi.repository import Gtk
 
-from pynicotine import slskmessages
 from pynicotine.config import config
 from pynicotine.gtkgui.utils import load_ui_elements
-from pynicotine.gtkgui.utils import triggers_context_menu
-from pynicotine.gtkgui.widgets.messagedialogs import entry_dialog
+from pynicotine.gtkgui.widgets.dialogs import entry_dialog
 from pynicotine.gtkgui.widgets.popupmenu import PopupMenu
 from pynicotine.gtkgui.widgets.theme import update_widget_visuals
 from pynicotine.gtkgui.widgets.treeview import initialise_columns
 from pynicotine.gtkgui.widgets.treeview import save_columns
-from pynicotine.gtkgui.widgets.treeview import set_treeview_selected_row
 from pynicotine.gtkgui.widgets.treeview import show_country_tooltip
 from pynicotine.gtkgui.widgets.treeview import show_user_status_tooltip
 from pynicotine.logfacility import log
@@ -78,8 +74,7 @@ class UserList:
 
         self.column_numbers = list(range(self.usersmodel.get_n_columns()))
         self.cols = cols = initialise_columns(
-            "buddy_list",
-            self.UserListTree,
+            "buddy_list", self.UserListTree,
             ["status", _("Status"), 25, "pixbuf", None],
             ["country", _("Country"), 25, "pixbuf", None],
             ["user", _("User"), 250, "text", None],
@@ -123,47 +118,16 @@ class UserList:
 
         self.UserListTree.set_model(self.usersmodel)
 
-        """ Buddy list """
+        """ Lists """
 
-        for user in config.sections["server"]["userlist"]:
-            try:
-                username, comment, notify, privileged, trusted, last_seen, country = user
-            except ValueError:
-                # Invalid user row
-                continue
-
-            try:
-                time_from_epoch = time.mktime(time.strptime(last_seen, "%m/%d/%Y %H:%M:%S"))
-            except ValueError:
-                last_seen = _("Never seen")
-                time_from_epoch = 0
-
-            username = str(username)
-            row = [
-                GObject.Value(GObject.TYPE_OBJECT, self.frame.get_status_image(0)),
-                GObject.Value(GObject.TYPE_OBJECT, self.frame.get_flag_image(country)),
-                username,
-                "",
-                "",
-                bool(trusted),
-                bool(notify),
-                bool(privileged),
-                str(last_seen),
-                str(comment),
-                0,
-                0,
-                0,
-                time_from_epoch,
-                str(country)
-            ]
-
-            self.user_iterators[username] = self.usersmodel.insert_with_valuesv(0, self.column_numbers, row)
+        for row in config.sections["server"]["userlist"]:
+            self.append_user_row(row)
 
         self.usersmodel.set_sort_column_id(2, Gtk.SortType.ASCENDING)
 
         self.buddies_combo_entries = (
-            self.frame.UserSearchCombo, self.frame.PrivateChatCombo, self.frame.UserInfoCombo, self.frame.UserBrowseCombo
-        )
+            self.frame.UserSearchCombo, self.frame.PrivateChatCombo, self.frame.UserInfoCombo,
+            self.frame.UserBrowseCombo)
 
         self.buddies_combos_fill()
 
@@ -171,7 +135,7 @@ class UserList:
 
         self.popup_menu_private_rooms = PopupMenu(self.frame)
 
-        self.popup_menu = popup = PopupMenu(frame)
+        self.popup_menu = popup = PopupMenu(frame, self.UserListTree, self.on_popup_menu)
         popup.setup_user_menu(page="userlist")
         popup.setup(
             ("", None),
@@ -180,20 +144,87 @@ class UserList:
             ("$" + _("_Trusted"), self.on_trusted),
             ("", None),
             (">" + _("Private Rooms"), self.popup_menu_private_rooms),
-            ("#" + _("Edit _Comments"), self.on_edit_comments),
+            ("#" + _("Edit _Comments..."), self.on_edit_comments),
             ("#" + _("_Remove"), self.on_remove_user)
         )
 
         self.update_visuals()
 
+    def append_user_row(self, row):
+
+        if not row or not isinstance(row, list):
+            return
+
+        username = str(row[0])
+
+        if not username:
+            return
+
+        try:
+            comment = str(row[1])
+        except IndexError:
+            comment = ""
+
+        try:
+            notify = bool(row[2])
+        except IndexError:
+            notify = False
+
+        try:
+            privileged = bool(row[3])
+        except IndexError:
+            privileged = False
+
+        try:
+            trusted = bool(row[4])
+        except IndexError:
+            trusted = False
+
+        try:
+            last_seen = str(row[5])
+        except IndexError:
+            last_seen = ""
+
+        try:
+            time_from_epoch = time.mktime(time.strptime(last_seen, "%m/%d/%Y %H:%M:%S"))
+        except ValueError:
+            last_seen = _("Never seen")
+            time_from_epoch = 0
+
+        try:
+            country = str(row[6])
+        except IndexError:
+            country = ""
+
+        row = [
+            GObject.Value(GObject.TYPE_OBJECT, self.frame.get_status_image(0)),
+            GObject.Value(GObject.TYPE_OBJECT, self.frame.get_flag_image(country)),
+            username,
+            "",
+            "",
+            trusted,
+            notify,
+            privileged,
+            last_seen,
+            comment,
+            0,
+            0,
+            0,
+            time_from_epoch,
+            country
+        ]
+
+        self.user_iterators[username] = self.usersmodel.insert_with_valuesv(0, self.column_numbers, row)
+
     def buddies_combos_fill(self):
 
         for widget in self.buddies_combo_entries:
             widget.remove_all()
             widget.append_text("")
 
-            for user in config.sections["server"]["userlist"]:
-                widget.append_text(str(user[0]))
+            for row in config.sections["server"]["userlist"]:
+                if row and isinstance(row, list):
+                    widget.append_text(str(row[0]))
 
     def on_tooltip(self, widget, x, y, keyboard_mode, tooltip):
 
@@ -208,13 +239,13 @@ class UserList:
 
     def on_add_user(self, widget, *args):
 
-        text = widget.get_text()
+        username = widget.get_text()
 
-        if not text:
+        if not username:
             return
 
         widget.set_text("")
-        self.add_to_list(text)
+        self.frame.np.userlist.add_user(username)
 
     def update_visuals(self):
 
@@ -270,22 +301,6 @@ class UserList:
             store.set_value(iterator, 9, comments)
             self.save_user_list()
 
-    def conn_close(self):
-
-        for i in self.usersmodel:
-            iterator = i.iter
-
-            self.usersmodel.set_value(iterator, 0, GObject.Value(GObject.TYPE_OBJECT, self.frame.get_status_image(0)))
-            self.usersmodel.set_value(iterator, 3, "")
-            self.usersmodel.set_value(iterator, 4, "")
-            self.usersmodel.set_value(iterator, 10, 0)
-            self.usersmodel.set_value(iterator, 11, 0)
-            self.usersmodel.set_value(iterator, 12, 0)
-
-            if not self.usersmodel.get_value(iterator, 8):
-                user = self.usersmodel.get_value(iterator, 2)
-                self.set_last_seen(user)
-
     def get_selected_username(self, treeview):
 
         model, iterator = treeview.get_selection().get_selected()
@@ -311,46 +326,34 @@ class UserList:
 
         return username, trusted, notify, privileged, status
 
-    def on_list_clicked(self, widget, event):
+    def on_row_activated(self, treeview, path, column):
 
-        if triggers_context_menu(event):
-            set_treeview_selected_row(widget, event)
-            return self.on_popup_menu(widget)
+        user = self.get_selected_username(treeview)
 
-        if event.type == Gdk.EventType._2BUTTON_PRESS:
-            user = self.get_selected_username(widget)
+        if user is not None:
+            self.frame.np.privatechats.show_user(user)
+            self.frame.change_main_page("private")
 
-            if user is not None:
-                self.frame.privatechats.send_message(user, show_user=True)
-                self.frame.change_main_page("private")
-                return True
-
-        return False
-
-    def on_popup_menu(self, widget):
+    def on_popup_menu(self, menu, widget):
 
         username, trusted, notify, privileged, status = self.get_selected_username_details(widget)
         if username is None:
-            return False
+            return True
 
-        self.popup_menu.set_user(username)
-        self.popup_menu.toggle_user_items()
-        self.popup_menu.populate_private_rooms(self.popup_menu_private_rooms)
+        menu.set_user(username)
+        menu.toggle_user_items()
+        menu.populate_private_rooms(self.popup_menu_private_rooms)
 
-        actions = self.popup_menu.get_actions()
+        actions = menu.get_actions()
 
         actions[_("Private Rooms")].set_enabled(
-            status or
-            self.popup_menu.user != config.sections["server"]["login"]
+            status or menu.user != config.sections["server"]["login"]
         )
 
         actions[_("_Online Notify")].set_state(GLib.Variant.new_boolean(notify))
         actions[_("_Privileged")].set_state(GLib.Variant.new_boolean(privileged))
         actions[_("_Trusted")].set_state(GLib.Variant.new_boolean(trusted))
 
-        self.popup_menu.popup()
-        return True
-
     def get_user_status(self, msg):
 
         status = msg.status
@@ -379,7 +382,7 @@ class UserList:
                 status_text = _("User %s is offline")
 
             log.add(status_text, user)
-            self.frame.notifications.new_notification(status_text % user)
+            self.frame.notifications.new_text_notification(status_text % user)
 
         img = self.frame.get_status_image(status)
         self.usersmodel.set_value(iterator, 0, GObject.Value(GObject.TYPE_OBJECT, img))
@@ -407,7 +410,7 @@ class UserList:
         self.usersmodel.set_value(iterator, 11, GObject.Value(GObject.TYPE_UINT64, msg.avgspeed))
         self.usersmodel.set_value(iterator, 12, GObject.Value(GObject.TYPE_UINT64, msg.files))
 
-    def set_user_flag(self, user, country):
+    def set_user_country(self, user, country):
 
         iterator = self.user_iterators.get(user)
 
@@ -417,7 +420,7 @@ class UserList:
         self.usersmodel.set_value(iterator, 1, GObject.Value(GObject.TYPE_OBJECT, self.frame.get_flag_image(country)))
         self.usersmodel.set_value(iterator, 14, "flag_" + country)
 
-    def add_to_list(self, user):
+    def add_user(self, user):
 
         if user in self.user_iterators:
             return
@@ -448,19 +451,13 @@ class UserList:
 
         self.save_user_list()
 
-        # Request user status, speed and number of shared files
-        self.frame.np.watch_user(user, force_update=True)
-
-        # Request user's IP address, so we can get the country
-        self.frame.np.queue.append(slskmessages.GetPeerAddress(user))
-
         for widget in self.buddies_combo_entries:
             widget.append_text(user)
 
         if config.sections["words"]["buddies"]:
             self.frame.update_completions()
 
-    def remove_from_list(self, user):
+    def remove_user(self, user):
 
         if user in self.user_iterators:
             self.usersmodel.remove(self.user_iterators[user])
@@ -477,11 +474,11 @@ class UserList:
         user_list = []
 
         for i in self.usersmodel:
-            status_icon, flag, user, hspeed, hfile_count, trusted, notify, privileged, hlast_seen, comments, status, speed, file_count, last_seen, country = i
+            (status_icon, flag, user, hspeed, hfile_count, trusted, notify, privileged,
+                hlast_seen, comments, status, speed, file_count, last_seen, country) = i
             user_list.append([user, comments, notify, privileged, trusted, hlast_seen, country])
 
-        config.sections["server"]["userlist"] = user_list
-        config.write_configuration()
+        self.frame.np.userlist.save_user_list(user_list)
 
     def save_columns(self):
         save_columns("buddy_list", self.UserListTree.get_columns())
@@ -553,12 +550,28 @@ class UserList:
 
         entry_dialog(
             parent=self.frame.MainWindow,
-            title=_("Edit comments") + "...",
-            message=_("Comments") + ":",
+            title=_("Edit Comments"),
+            message=_("Add a few notes associated with user %s:") % user,
             callback=self.on_edit_comments_response,
             callback_data=user,
             default=comments
         )
 
     def on_remove_user(self, *args):
-        self.remove_from_list(self.popup_menu.get_user())
+        self.frame.np.userlist.remove_user(self.popup_menu.get_user())
+
+    def server_disconnect(self):
+
+        for i in self.usersmodel:
+            iterator = i.iter
+
+            self.usersmodel.set_value(iterator, 0, GObject.Value(GObject.TYPE_OBJECT, self.frame.get_status_image(0)))
+            self.usersmodel.set_value(iterator, 3, "")
+            self.usersmodel.set_value(iterator, 4, "")
+            self.usersmodel.set_value(iterator, 10, 0)
+            self.usersmodel.set_value(iterator, 11, 0)
+            self.usersmodel.set_value(iterator, 12, 0)
+
+            if not self.usersmodel.get_value(iterator, 8):
+                user = self.usersmodel.get_value(iterator, 2)
+                self.set_last_seen(user)
