@@ -22,7 +22,6 @@
 
 import os
 import sys
-import threading
 
 from ast import literal_eval
 from time import time
@@ -38,18 +37,18 @@ returncode = {
 }                # returning nothing is the same as 'pass'
 
 
-class PluginHandler(object):
+class PluginHandler:
 
-    def __init__(self, frame, config):
-        self.frame = frame
+    def __init__(self, core, config):
+
+        self.core = core
         self.config = config
 
-        log.add(_("Loading plugin handler"))
+        log.add_debug("Loading plugin handler")
 
         self.my_username = self.config.sections["server"]["login"]
         self.plugindirs = []
         self.enabled_plugins = {}
-        self.loaded_plugins = {}
 
         try:
             os.makedirs(config.plugin_dir)
@@ -68,82 +67,104 @@ class PluginHandler(object):
         else:
             log.add(_("It appears '%s' is not a directory, not loading plugins."), config.plugin_dir)
 
-    def __findplugin(self, pluginname):
+    def update_completions(self, plugin):
+
+        if plugin.__publiccommands__ and self.config.sections["words"]["commands"]:
+            self.core.chatrooms.update_completions()
+
+        if plugin.__privatecommands__ and self.config.sections["words"]["commands"]:
+            self.core.privatechats.update_completions()
+
+    def __findplugin(self, plugin_name):
         for directory in self.plugindirs:
-            fullpath = os.path.join(directory, pluginname)
+            fullpath = os.path.join(directory, plugin_name)
 
             if os.path.exists(fullpath):
                 return fullpath
 
         return None
 
-    def toggle_plugin(self, pluginname):
+    def toggle_plugin(self, plugin_name):
 
-        enabled = pluginname in self.enabled_plugins
+        enabled = plugin_name in self.enabled_plugins
 
         if enabled:
-            thread = threading.Thread(target=self.disable_plugin, args=(pluginname,))
+            self.disable_plugin(plugin_name)
         else:
-            thread = threading.Thread(target=self.enable_plugin, args=(pluginname,))
+            self.enable_plugin(plugin_name)
 
-        thread.name = "TogglePlugin"
-        thread.daemon = True
-        thread.start()
-
-    def load_plugin(self, pluginname):
+    def load_plugin(self, plugin_name):
 
         try:
             # Import builtin plugin
             from importlib import import_module
-            plugin = import_module("pynicotine.plugins." + pluginname)
+            plugin = import_module("pynicotine.plugins." + plugin_name)
 
         except Exception:
             # Import user plugin
-            path = self.__findplugin(pluginname)
+            path = self.__findplugin(plugin_name)
 
             if path is None:
-                log.add(_("Failed to load plugin '%s', could not find it."), pluginname)
+                log.add(_("Failed to load plugin '%s', could not find it."), plugin_name)
                 return False
 
             import importlib.util
-            spec = importlib.util.spec_from_file_location(pluginname, os.path.join(path, '__init__.py'))
+            spec = importlib.util.spec_from_file_location(plugin_name, os.path.join(path, '__init__.py'))
             plugin = importlib.util.module_from_spec(spec)
             spec.loader.exec_module(plugin)
 
-        instance = plugin.Plugin(self, enable_plugin=False)
-        self.plugin_settings(pluginname, instance)
-        instance.LoadNotification()
+        instance = plugin.Plugin(self, self.config, self.core)
+        self.plugin_settings(plugin_name, instance)
 
-        self.loaded_plugins[pluginname] = plugin
-        return plugin
+        if hasattr(plugin, "enable"):
+            instance.log("top-level enable() function is obsolete, please use BasePlugin.init() instead" % plugin_name)
 
-    def enable_plugin(self, pluginname):
+        if hasattr(plugin, "disable"):
+            instance.log("top-level disable() function is obsolete, please use BasePlugin.disable() instead"
+                         % plugin_name)
+
+        if hasattr(instance, "LoadNotification"):
+            instance.log("LoadNotification() is obsolete, please use init()")
+
+        return instance
+
+    def enable_plugin(self, plugin_name):
 
         # Our config file doesn't play nicely with some characters
-        if "=" in pluginname:
-            log.add(_("Unable to enable plugin %(name)s. Plugin folder name contains invalid characters: %(characters)s"), {
-                "name": pluginname,
-                "characters": "="
-            })
-            return
+        if "=" in plugin_name:
+            log.add(
+                _("Unable to enable plugin %(name)s. Plugin folder name contains invalid characters: %(characters)s"), {
+                    "name": plugin_name,
+                    "characters": "="
+                })
+            return False
 
-        if pluginname in self.enabled_plugins:
-            return
+        if plugin_name in self.enabled_plugins:
+            return False
 
         try:
-            plugin = self.load_plugin(pluginname)
+            plugin = self.load_plugin(plugin_name)
 
             if not plugin:
-                raise Exception("Error loading plugin '%s'" % pluginname)
+                raise Exception("Error loading plugin '%s'" % plugin_name)
 
-            plugin.enable(self)
-            self.enabled_plugins[pluginname] = plugin
-            log.add(_("Enabled plugin %s"), plugin.PLUGIN.__name__)
+            plugin.init()
+
+            for trigger, _func in plugin.__publiccommands__:
+                self.core.chatrooms.CMDS.add('/' + trigger + ' ')
+
+            for trigger, _func in plugin.__privatecommands__:
+                self.core.privatechats.CMDS.add('/' + trigger + ' ')
+
+            self.update_completions(plugin)
+
+            self.enabled_plugins[plugin_name] = plugin
+            log.add(_("Enabled plugin %s"), plugin.__name__)
 
         except Exception:
-            from traceback import print_exc
-            print_exc()
-            log.add(_("Unable to enable plugin %s"), pluginname)
+            from traceback import format_exc
+            log.add(_("Unable to enable plugin %(module)s\n%(exc_trace)s"),
+                    {'module': plugin_name, 'exc_trace': format_exc()})
             return False
 
         return True
@@ -159,39 +180,51 @@ class PluginHandler(object):
 
         return pluginlist
 
-    def disable_plugin(self, pluginname):
-        if pluginname not in self.enabled_plugins:
-            return
+    def disable_plugin(self, plugin_name):
+        if plugin_name not in self.enabled_plugins:
+            return False
 
         try:
-            plugin = self.enabled_plugins[pluginname]
+            plugin = self.enabled_plugins[plugin_name]
 
-            log.add(_("Disabled plugin {}".format(plugin.PLUGIN.__name__)))
-            del self.enabled_plugins[pluginname]
-            plugin.disable(self)
+            log.add(_("Disabled plugin {}".format(plugin.__name__)))
+            plugin.disable()
+
+            for trigger, _func in plugin.__publiccommands__:
+                self.core.chatrooms.CMDS.remove('/' + trigger + ' ')
+
+            for trigger, _func in plugin.__privatecommands__:
+                self.core.privatechats.CMDS.remove('/' + trigger + ' ')
+
+            self.update_completions(plugin)
+
+            del self.enabled_plugins[plugin_name]
+            del plugin
 
         except Exception:
-            from traceback import print_exc
-            print_exc()
-            log.add(_("Unable to fully disable plugin %s"), pluginname)
+            from traceback import format_exc
+            log.add(_("Unable to fully disable plugin %(module)s\n%(exc_trace)s"),
+                    {'module': plugin_name, 'exc_trace': format_exc()})
             return False
 
         return True
 
-    def get_plugin_settings(self, pluginname):
-        if pluginname in self.enabled_plugins:
-            plugin = self.enabled_plugins[pluginname]
+    def get_plugin_settings(self, plugin_name):
+        if plugin_name in self.enabled_plugins:
+            plugin = self.enabled_plugins[plugin_name]
 
-            if hasattr(plugin.PLUGIN, "metasettings"):
-                return plugin.PLUGIN.metasettings
+            if plugin.metasettings:
+                return plugin.metasettings
 
-    def get_plugin_info(self, pluginname):
-        path = os.path.join(self.__findplugin(pluginname), 'PLUGININFO')
+        return None
 
-        with open(path) as f:
+    def get_plugin_info(self, plugin_name):
+        path = os.path.join(self.__findplugin(plugin_name), 'PLUGININFO')
+
+        with open(path) as file_handle:
             infodict = {}
 
-            for line in f:
+            for line in file_handle:
                 try:
                     key, val = line.split("=", 1)
                     infodict[key.strip()] = literal_eval(val.strip())
@@ -212,24 +245,21 @@ class PluginHandler(object):
         to_enable = self.config.sections["plugins"]["enabled"]
 
         for plugin in to_enable:
-            thread = threading.Thread(target=self.enable_plugin, args=(plugin,))
-            thread.name = "EnablePlugin"
-            thread.daemon = True
-            thread.start()
+            self.enable_plugin(plugin)
 
-    def plugin_settings(self, pluginname, plugin):
+    def plugin_settings(self, plugin_name, plugin):
         try:
-            if not hasattr(plugin, "settings"):
+            if not plugin.settings:
                 return
 
-            if pluginname not in self.config.sections["plugins"]:
-                self.config.sections["plugins"][pluginname] = plugin.settings
+            if plugin_name not in self.config.sections["plugins"]:
+                self.config.sections["plugins"][plugin_name] = plugin.settings
 
             for i in plugin.settings:
-                if i not in self.config.sections["plugins"][pluginname]:
-                    self.config.sections["plugins"][pluginname][i] = plugin.settings[i]
+                if i not in self.config.sections["plugins"][plugin_name]:
+                    self.config.sections["plugins"][plugin_name][i] = plugin.settings[i]
 
-            customsettings = self.config.sections["plugins"][pluginname]
+            customsettings = self.config.sections["plugins"][plugin_name]
 
             for key in customsettings:
                 if key in plugin.settings:
@@ -251,13 +281,20 @@ class PluginHandler(object):
 
         for module, plugin in self.enabled_plugins.items():
             try:
-                if plugin.PLUGIN is None:
+                if plugin is None:
                     continue
 
+                return_value = None
+
                 if public_command:
-                    return_value = plugin.PLUGIN.PublicCommandEvent(command, source, args)
+                    for trigger, func in plugin.__publiccommands__:
+                        if trigger == command:
+                            return_value = func(plugin, source, args)
+
                 else:
-                    return_value = plugin.PLUGIN.PrivateCommandEvent(command, source, args)
+                    for trigger, func in plugin.__privatecommands__:
+                        if trigger == command:
+                            return_value = func(plugin, source, args)
 
                 if return_value is None:
                     # Nothing changed, continue to the next plugin
@@ -265,17 +302,20 @@ class PluginHandler(object):
 
                 if return_value == returncode['zap']:
                     return True
-                elif return_value == returncode['pass']:
+
+                if return_value == returncode['pass']:
                     continue
 
-                log.add(_("Plugin %(module)s returned something weird, '%(value)s', ignoring"), {'module': module, 'value': str(return_value)})
+                log.add(_("Plugin %(module)s returned something weird, '%(value)s', ignoring"),
+                        {'module': module, 'value': str(return_value)})
 
             except Exception:
                 from traceback import extract_stack
                 from traceback import extract_tb
                 from traceback import format_list
 
-                log.add(_("Plugin %(module)s failed with error %(errortype)s: %(error)s.\nTrace: %(trace)s\nProblem area:%(area)s"), {
+                log.add(_("Plugin %(module)s failed with error %(errortype)s: %(error)s.\n"
+                          "Trace: %(trace)s\nProblem area:%(area)s"), {
                     'module': module,
                     'errortype': sys.exc_info()[0],
                     'error': sys.exc_info()[1],
@@ -285,14 +325,23 @@ class PluginHandler(object):
 
         return False
 
-    def trigger_event(self, function, args):
+    def trigger_event(self, function_name, args):
         """ Triggers an event for the plugins. Since events and notifications
         are precisely the same except for how n+ responds to them, both can be
         triggered by this function. """
 
+        function_name_camelcase = ''.join(x.capitalize() or '_' for x in function_name.split('_'))
+
         for module, plugin in self.enabled_plugins.items():
             try:
-                return_value = getattr(plugin.PLUGIN, function)(*args)
+                if hasattr(plugin, function_name_camelcase):
+                    plugin.log("%(old_function)s is deprecated, please use %(new_function)s" % {
+                        "old_function": function_name_camelcase,
+                        "new_function": function_name
+                    })
+                    return_value = getattr(plugin, function_name_camelcase)(*args)
+                else:
+                    return_value = getattr(plugin, function_name)(*args)
 
                 if return_value is None:
                     # Nothing changed, continue to the next plugin
@@ -305,19 +354,23 @@ class PluginHandler(object):
 
                 if return_value == returncode['zap']:
                     return None
-                elif return_value == returncode['break']:
+
+                if return_value == returncode['break']:
                     return args
-                elif return_value == returncode['pass']:
+
+                if return_value == returncode['pass']:
                     continue
 
-                log.add(_("Plugin %(module)s returned something weird, '%(value)s', ignoring"), {'module': module, 'value': return_value})
+                log.add(_("Plugin %(module)s returned something weird, '%(value)s', ignoring"),
+                        {'module': module, 'value': return_value})
 
             except Exception:
                 from traceback import extract_stack
                 from traceback import extract_tb
                 from traceback import format_list
 
-                log.add(_("Plugin %(module)s failed with error %(errortype)s: %(error)s.\nTrace: %(trace)s\nProblem area:%(area)s"), {
+                log.add(_("Plugin %(module)s failed with error %(errortype)s: %(error)s.\n"
+                          "Trace: %(trace)s\nProblem area:%(area)s"), {
                     'module': module,
                     'errortype': sys.exc_info()[0],
                     'error': sys.exc_info()[1],
@@ -328,109 +381,105 @@ class PluginHandler(object):
         return args
 
     def search_request_notification(self, searchterm, user, searchid):
-        self.trigger_event("SearchRequestNotification", (searchterm, user, searchid))
+        self.trigger_event("search_request_notification", (searchterm, user, searchid))
 
     def distrib_search_notification(self, searchterm, user, searchid):
-        self.trigger_event("DistribSearchNotification", (searchterm, user, searchid))
+        self.trigger_event("distrib_search_notification", (searchterm, user, searchid))
 
     def public_room_message_notification(self, room, user, line):
-        self.trigger_event("PublicRoomMessageNotification", (room, user, line))
+        self.trigger_event("public_room_message_notification", (room, user, line))
 
     def incoming_private_chat_event(self, user, line):
         if user != self.my_username:
             # dont trigger the scripts on our own talking - we've got "Outgoing" for that
-            return self.trigger_event("IncomingPrivateChatEvent", (user, line))
-        else:
-            return (user, line)
+            return self.trigger_event("incoming_private_chat_event", (user, line))
+
+        return (user, line)
 
     def incoming_private_chat_notification(self, user, line):
-        self.trigger_event("IncomingPrivateChatNotification", (user, line))
+        self.trigger_event("incoming_private_chat_notification", (user, line))
 
     def incoming_public_chat_event(self, room, user, line):
-        return self.trigger_event("IncomingPublicChatEvent", (room, user, line))
+        return self.trigger_event("incoming_public_chat_event", (room, user, line))
 
     def incoming_public_chat_notification(self, room, user, line):
-        self.trigger_event("IncomingPublicChatNotification", (room, user, line))
+        self.trigger_event("incoming_public_chat_notification", (room, user, line))
 
     def outgoing_private_chat_event(self, user, line):
         if line is not None:
             # if line is None nobody actually said anything
-            return self.trigger_event("OutgoingPrivateChatEvent", (user, line))
-        else:
-            return (user, line)
+            return self.trigger_event("outgoing_private_chat_event", (user, line))
+
+        return (user, line)
 
     def outgoing_private_chat_notification(self, user, line):
-        self.trigger_event("OutgoingPrivateChatNotification", (user, line))
+        self.trigger_event("outgoing_private_chat_notification", (user, line))
 
     def outgoing_public_chat_event(self, room, line):
-        return self.trigger_event("OutgoingPublicChatEvent", (room, line))
+        return self.trigger_event("outgoing_public_chat_event", (room, line))
 
     def outgoing_public_chat_notification(self, room, line):
-        self.trigger_event("OutgoingPublicChatNotification", (room, line))
+        self.trigger_event("outgoing_public_chat_notification", (room, line))
 
     def outgoing_global_search_event(self, text):
-        return self.trigger_event("OutgoingGlobalSearchEvent", (text,))
+        return self.trigger_event("outgoing_global_search_event", (text,))
 
     def outgoing_room_search_event(self, rooms, text):
-        return self.trigger_event("OutgoingRoomSearchEvent", (rooms, text))
+        return self.trigger_event("outgoing_room_search_event", (rooms, text))
 
     def outgoing_buddy_search_event(self, text):
-        return self.trigger_event("OutgoingBuddySearchEvent", (text,))
+        return self.trigger_event("outgoing_buddy_search_event", (text,))
 
     def outgoing_user_search_event(self, users, text):
-        return self.trigger_event("OutgoingUserSearchEvent", (users, text))
+        return self.trigger_event("outgoing_user_search_event", (users, text))
 
-    def user_resolve_notification(self, user, ip, port, country=None):
+    def user_resolve_notification(self, user, ip_address, port, country=None):
         """Notification for user IP:Port resolving.
 
         Note that country is only set when the user requested the resolving"""
-        self.trigger_event("UserResolveNotification", (user, ip, port, country))
+        self.trigger_event("user_resolve_notification", (user, ip_address, port, country))
 
     def server_connect_notification(self):
-        self.trigger_event("ServerConnectNotification", (),)
+        self.trigger_event("server_connect_notification", (),)
 
     def server_disconnect_notification(self, userchoice):
-        self.trigger_event("ServerDisconnectNotification", (userchoice, ))
+        self.trigger_event("server_disconnect_notification", (userchoice, ))
 
     def join_chatroom_notification(self, room):
-        self.trigger_event("JoinChatroomNotification", (room,))
+        self.trigger_event("join_chatroom_notification", (room,))
 
     def leave_chatroom_notification(self, room):
-        self.trigger_event("LeaveChatroomNotification", (room,))
+        self.trigger_event("leave_chatroom_notification", (room,))
 
     def user_join_chatroom_notification(self, room, user):
-        self.trigger_event("UserJoinChatroomNotification", (room, user,))
+        self.trigger_event("user_join_chatroom_notification", (room, user,))
 
     def user_leave_chatroom_notification(self, room, user):
-        self.trigger_event("UserLeaveChatroomNotification", (room, user,))
-
-    def upload_queued_notification(self, user, virtualfile, realfile):
-        self.trigger_event("UploadQueuedNotification", (user, virtualfile, realfile))
+        self.trigger_event("user_leave_chatroom_notification", (room, user,))
 
     def user_stats_notification(self, user, stats):
-        self.trigger_event("UserStatsNotification", (user, stats))
+        self.trigger_event("user_stats_notification", (user, stats))
+
+    def upload_queued_notification(self, user, virtual_path, real_path):
+        self.trigger_event("upload_queued_notification", (user, virtual_path, real_path))
+
+    def upload_started_notification(self, user, virtual_path, real_path):
+        self.trigger_event("upload_started_notification", (user, virtual_path, real_path))
+
+    def upload_finished_notification(self, user, virtual_path, real_path):
+        self.trigger_event("upload_finished_notification", (user, virtual_path, real_path))
+
+    def download_started_notification(self, user, virtual_path, real_path):
+        self.trigger_event("download_started_notification", (user, virtual_path, real_path))
+
+    def download_finished_notification(self, user, virtual_path, real_path):
+        self.trigger_event("download_finished_notification", (user, virtual_path, real_path))
 
     def shutdown_notification(self):
-        self.trigger_event("ShutdownNotification", (),)
-
-    """ Other Functions """
-
-    def log(self, text):
-        log.add(text)
-
-    def saychatroom(self, room, text):
-        self.frame.np.queue.append(slskmessages.SayChatroom(room, text))
-
-    def sayprivate(self, user, text):
-        '''Send user message in private (showing up in GUI)'''
-        self.frame.privatechats.users[user].send_message(text)
-
-    def sendprivate(self, user, text):
-        '''Send user message in private (not showing up in GUI)'''
-        self.frame.privatechats.send_message(user, text)
+        self.trigger_event("shutdown_notification", (),)
 
 
-class ResponseThrottle(object):
+class ResponseThrottle:
 
     """
     ResponseThrottle - Mutnick 2016
@@ -444,13 +493,19 @@ class ResponseThrottle(object):
     Some of the throttle logic is guesswork as server code is closed source, but works adequately.
     """
 
-    def __init__(self, frame, plugin_name, logging=False):
-        self.frame = frame
+    def __init__(self, core, plugin_name, logging=False):
+
+        self.core = core
         self.plugin_name = plugin_name
         self.logging = logging
         self.plugin_usage = {}
 
+        self.room = None
+        self.nick = None
+        self.request = None
+
     def ok_to_respond(self, room, nick, request, seconds_limit_min=30):
+
         self.room = room
         self.nick = nick
         self.request = request
@@ -467,14 +522,14 @@ class ResponseThrottle(object):
 
         port = False
         try:
-            ip, port = self.frame.np.users[nick].addr
+            _ip_address, port = self.core.users[nick].addr
         except Exception:
             port = True
 
-        if self.frame.np.network_filter.is_user_ignored(nick):
+        if self.core.network_filter.is_user_ignored(nick):
             willing_to_respond, reason = False, "The nick is ignored"
 
-        elif self.frame.np.network_filter.is_user_ip_ignored(nick):
+        elif self.core.network_filter.is_user_ip_ignored(nick):
             willing_to_respond, reason = False, "The nick's Ip is ignored"
 
         elif not port:
@@ -484,15 +539,15 @@ class ResponseThrottle(object):
             if (current_time - last_time) < 12 * seconds_limit_min:
                 willing_to_respond, reason = False, "Too soon for same nick to request same resource in room"
 
-        elif (request == last_request):
+        elif request == last_request:
             if (current_time - last_time) < 3 * seconds_limit_min:
                 willing_to_respond, reason = False, "Too soon for different nick to request same resource in room"
 
         else:
             recent_responses = 0
 
-            for responded_room in self.plugin_usage:
-                if (current_time - self.plugin_usage[responded_room]['last_time']) < seconds_limit_min:
+            for responded_room, room_dict in self.plugin_usage.items():
+                if (current_time - room_dict['last_time']) < seconds_limit_min:
                     recent_responses += 1
 
                     if responded_room == room:
@@ -502,163 +557,177 @@ class ResponseThrottle(object):
             if recent_responses > 3:
                 willing_to_respond, reason = False, "Responded in multiple rooms enough"
 
-        if self.logging:
-            if not willing_to_respond:
-                base_log_msg = "{} plugin request rejected - room '{}', nick '{}'".format(self.plugin_name, room, nick)
-                log.add_debug("{} - {}".format(base_log_msg, reason))
+        if self.logging and not willing_to_respond:
+            base_log_msg = "{} plugin request rejected - room '{}', nick '{}'".format(self.plugin_name, room, nick)
+            log.add_debug("{} - {}".format(base_log_msg, reason))
 
         return willing_to_respond
 
-    def responded(self, msg=""):
+    def responded(self):
         # possible TODO's: we could actually say public the msg here
         # make more stateful - track past msg's as additional responder willingness criteria, etc
         self.plugin_usage[self.room] = {'last_time': time(), 'last_request': self.request, 'last_nick': self.nick}
 
 
-class BasePlugin(object):
+class BasePlugin:
 
     __name__ = "BasePlugin"
-    __desc__ = "No description provided"
-    __version__ = "2016-08-30"
     __publiccommands__ = []
     __privatecommands__ = []
+    settings = {}
+    metasettings = {}
 
-    def __init__(self, parent, enable_plugin=True):
+    def __init__(self, parent, config, core):
 
         # Never override this function, override init() instead
         self.parent = parent
-        self.frame = parent.frame
-
-        if not enable_plugin:
-            # Plugin was loaded, but not enabled yet
-            return
-
-        self.init()
-        for (trigger, func) in self.__publiccommands__:
-            self.frame.chatrooms.CMDS.add('/' + trigger + ' ')
-        for (trigger, func) in self.__privatecommands__:
-            self.frame.privatechats.CMDS.add('/' + trigger + ' ')
+        self.config = config
+        self.core = core
+        self.frame = core.ui_callback
 
     def init(self):
-        # Custom init function for plugins
+        # Custom enable function for plugins
         pass
 
-    def LoadSettings(self, settings):  # noqa
-        self.settings = settings
-
-    def LoadNotification(self):  # noqa
+    def disable(self):
+        # Custom disable function for plugins
         pass
 
-    def PublicRoomMessageNotification(self, room, user, line):  # noqa
+    def public_room_message_notification(self, room, user, line):
         pass
 
-    def SearchRequestNotification(self, searchterm, user, searchid):  # noqa
+    def search_request_notification(self, searchterm, user, searchid):
         pass
 
-    def DistribSearchNotification(self, searchterm, user, searchid):  # noqa
+    def distrib_search_notification(self, searchterm, user, searchid):
         pass
 
-    def IncomingPrivateChatEvent(self, user, line):  # noqa
+    def incoming_private_chat_event(self, user, line):
         pass
 
-    def IncomingPrivateChatNotification(self, user, line):  # noqa
+    def incoming_private_chat_notification(self, user, line):
         pass
 
-    def IncomingPublicChatEvent(self, room, user, line):  # noqa
+    def incoming_public_chat_event(self, room, user, line):
         pass
 
-    def IncomingPublicChatNotification(self, room, user, line):  # noqa
+    def incoming_public_chat_notification(self, room, user, line):
         pass
 
-    def OutgoingPrivateChatEvent(self, user, line):  # noqa
+    def outgoing_private_chat_event(self, user, line):
         pass
 
-    def OutgoingPrivateChatNotification(self, user, line):  # noqa
+    def outgoing_private_chat_notification(self, user, line):
         pass
 
-    def OutgoingPublicChatEvent(self, room, line):  # noqa
+    def outgoing_public_chat_event(self, room, line):
         pass
 
-    def OutgoingPublicChatNotification(self, room, line):  # noqa
+    def outgoing_public_chat_notification(self, room, line):
         pass
 
-    def OutgoingGlobalSearchEvent(self, text):  # noqa
+    def outgoing_global_search_event(self, text):
         pass
 
-    def OutgoingRoomSearchEvent(self, rooms, text):  # noqa
+    def outgoing_room_search_event(self, rooms, text):
         pass
 
-    def OutgoingBuddySearchEvent(self, text):  # noqa
+    def outgoing_buddy_search_event(self, text):
         pass
 
-    def OutgoingUserSearchEvent(self, users, text):  # noqa
+    def outgoing_user_search_event(self, users, text):
         pass
 
-    def UserResolveNotification(self, user, ip, port, country):  # noqa
+    def user_resolve_notification(self, user, ip_address, port, country):
         pass
 
-    def ServerConnectNotification(self):  # noqa
+    def server_connect_notification(self):
         pass
 
-    def ServerDisconnectNotification(self, userchoice):  # noqa
+    def server_disconnect_notification(self, userchoice):
         pass
 
-    def JoinChatroomNotification(self, room):  # noqa
+    def join_chatroom_notification(self, room):
         pass
 
-    def LeaveChatroomNotification(self, room):  # noqa
+    def leave_chatroom_notification(self, room):
         pass
 
-    def UserJoinChatroomNotification(self, room, user):  # noqa
+    def user_join_chatroom_notification(self, room, user):
         pass
 
-    def UserLeaveChatroomNotification(self, room, user):  # noqa
+    def user_leave_chatroom_notification(self, room, user):
         pass
 
-    def UploadQueuedNotification(self, user, virtualfile, realfile):  # noqa
+    def user_stats_notification(self, user, stats):
         pass
 
-    def UserStatsNotification(self, user, stats):  # noqa
+    def upload_queued_notification(self, user, virtual_path, real_path):
         pass
 
-    def ShutdownNotification(self):  # noqa
+    def upload_started_notification(self, user, virtual_path, real_path):
+        pass
+
+    def upload_finished_notification(self, user, virtual_path, real_path):
+        pass
+
+    def download_started_notification(self, user, virtual_path, real_path):
+        pass
+
+    def download_finished_notification(self, user, virtual_path, real_path):
+        pass
+
+    def shutdown_notification(self):
         pass
 
     # The following are functions to make your life easier,
     # you shouldn't override them.
-    def log(self, text):
-        self.parent.log(self.__name__ + ": " + text)
-
     def saypublic(self, room, text):
-        self.parent.saychatroom(room, text)
+        self.log("saypublic(room, text) is deprecated, please use send_public(room, text)")
+        self.send_public(room, text)
 
     def sayprivate(self, user, text):
-        '''Send user message in private (shows up in GUI)'''
-        self.parent.sayprivate(user, text)
+        self.log("sayprivate(user, text) is deprecated, please use send_private(user, text)")
+        self.send_private(user, text, show_ui=True)
 
     def sendprivate(self, user, text):
-        '''Send user message in private (doesn't show up in GUI)'''
-        self.parent.sendprivate(user, text)
+        self.log("sendprivate(user, text) is deprecated, please use send_private(user, text, show_ui=False)")
+        self.send_private(user, text, show_ui=False)
 
     def fakepublic(self, room, user, text):
-        try:
-            room = self.frame.chatrooms.joinedrooms[room]
-        except KeyError:
-            return False
+
+        self.log("fakepublic(room, user, text) is deprecated, please use echo_public(room, text)")
 
         msg = slskmessages.SayChatroom(room, text)
         msg.user = user
-        room.say_chat_room(msg, text)
-        return True
+        self.core.chatrooms.say_chat_room(msg)
 
-    # The following are functions used by the plugin system,
-    # you are not allowed to override these.
-    def PublicCommandEvent(self, command, room, args):  # noqa
-        for (trigger, func) in self.__publiccommands__:
-            if trigger == command:
-                return func(self, room, args)
+    def log(self, msg, msg_args=None):
+        log.add(self.__name__ + ": " + msg, msg_args)
 
-    def PrivateCommandEvent(self, command, user, args):  # noqa
-        for (trigger, func) in self.__privatecommands__:
-            if trigger == command:
-                return func(self, user, args)
+    def send_public(self, room, text):
+        self.core.queue.append(slskmessages.SayChatroom(room, text))
+
+    def send_private(self, user, text, show_ui=True):
+        """ Send user message in private.
+        show_ui controls if the UI opens a private chat view for the user. """
+
+        if show_ui:
+            self.core.privatechats.show_user(user)
+
+        self.core.privatechats.send_message(user, text)
+
+    def echo_public(self, room, text, message_type="local"):
+        """ Display a raw message in chat rooms (not sent to others).
+        message_type changes the type (and color) of the message in the UI.
+        available message_type values: action, remote, local, hilite """
+
+        self.core.chatrooms.echo_message(room, text, message_type)
+
+    def echo_private(self, user, text, message_type="local"):
+        """ Display a raw message in private (not sent to others).
+        message_type changes the type (and color) of the message in the UI.
+        available message_type values: action, remote, local, hilite """
+
+        self.core.privatechats.show_user(user)
+        self.core.privatechats.echo_message(user, text, message_type)
