@@ -31,19 +31,105 @@ from pynicotine.logfacility import log
 
 class Search:
 
-    def __init__(self, np, config, queue, share_dbs, ui_callback=None):
+    def __init__(self, core, config, queue, share_dbs, geoip, ui_callback=None):
 
-        self.np = np
+        self.core = core
         self.config = config
         self.queue = queue
-        self.ui_callback = ui_callback
+        self.ui_callback = None
         self.searchid = int(random.random() * (2 ** 31 - 1))
         self.share_dbs = share_dbs
+        self.geoip = geoip
         self.translatepunctuation = str.maketrans(dict.fromkeys(string.punctuation, ' '))
 
+        if hasattr(ui_callback, "searches"):
+            self.ui_callback = ui_callback.searches
+
+    def request_folder_download(self, user, folder, visible_files):
+
+        # First queue the visible search results
+        if self.config.sections["transfers"]["reverseorder"]:
+            visible_files.sort(key=lambda x: x[1], reverse=True)
+
+        for file in visible_files:
+            user, fullpath, destination, size, bitrate, length = file
+
+            self.core.transfers.get_file(
+                user, fullpath, destination,
+                size=size, bitrate=bitrate, length=length, checkduplicate=True
+            )
+
+        # Ask for the rest of the files in the folder
+        self.core.transfers.get_folder(user, folder)
+
     """ Outgoing search requests """
 
-    def do_search(self, text, mode, users=[], room=None):
+    @staticmethod
+    def add_allowed_search_id(search_id):
+        """ Allow parsing search result messages for a search ID """
+        slskmessages.SEARCH_TOKENS_ALLOWED.add(search_id)
+
+    @staticmethod
+    def remove_allowed_search_id(search_id):
+        """ Disallow parsing search result messages for a search ID """
+        slskmessages.SEARCH_TOKENS_ALLOWED.discard(search_id)
+
+    def process_search_term(self, text, mode, room, user):
+
+        users = []
+        feedback = None
+
+        if mode == "global":
+            if self.core:
+                feedback = self.core.pluginhandler.outgoing_global_search_event(text)
+
+                if feedback is not None:
+                    text = feedback[0]
+
+        elif mode == "rooms":
+            # Space after Joined Rooms is important, so it doesn't conflict
+            # with any possible real room
+            if room == _("Joined Rooms ") or room.isspace():
+                room = None
+
+            if self.core:
+                feedback = self.core.pluginhandler.outgoing_room_search_event(room, text)
+
+                if feedback is not None:
+                    room, text = feedback
+
+        elif mode == "buddies" and self.core:
+            feedback = self.core.pluginhandler.outgoing_buddy_search_event(text)
+
+            if feedback is not None:
+                text = feedback[0]
+
+        elif mode == "user":
+            if user:
+                users = [user]
+            else:
+                return None
+
+            if self.core:
+                feedback = self.core.pluginhandler.outgoing_user_search_event(users, text)
+
+                if feedback is not None:
+                    users, text = feedback
+
+        else:
+            log.add("Unknown search mode, not using plugin system. Fix me!")
+
+        return text, room, users
+
+    def do_search(self, text, mode, room=None, user=None):
+
+        # Validate search term and run it through plugins
+        processed_search = self.process_search_term(text, mode, room, user)
+
+        if not processed_search:
+            return None
+
+        text, room, users = processed_search
 
         # Get a new search ID
         self.increment_search_id()
@@ -53,14 +139,18 @@ class Search:
         searchterm_words_ignore = (p[1:] for p in searchterm_words if p.startswith('-') and len(p) > 1)
 
         # Remove words starting with "-", results containing these are excluded by us later
-        searchterm_without_excluded = re.sub(r'(\s)-\w+', '', text)
+        searchterm_without_excluded = ' '.join(p for p in searchterm_words if not p.startswith('-'))
 
         if self.config.sections["searches"]["remove_special_chars"]:
             """
             Remove special characters from search term
             SoulseekQt doesn't seem to send search results if special characters are included (July 7, 2020)
             """
-            searchterm_without_excluded = re.sub(r'\W+', ' ', searchterm_without_excluded)
+            stripped_searchterm = re.sub(r'\W+', ' ', searchterm_without_excluded)
+
+            # Only modify search term if string also contains non-special characters
+            if stripped_searchterm:
+                searchterm_without_excluded = stripped_searchterm
 
         # Remove trailing whitespace
         searchterm_without_excluded = searchterm_without_excluded.strip()
@@ -94,36 +184,39 @@ class Search:
         elif mode == "user":
             self.do_peer_search(self.searchid, searchterm_without_excluded, users)
 
-        return self.searchid, searchterm_with_excluded, searchterm_without_excluded
+        self.add_allowed_search_id(self.searchid)
+        return (self.searchid, searchterm_with_excluded, searchterm_without_excluded)
 
-    def do_global_search(self, id, text):
-        self.queue.append(slskmessages.FileSearch(id, text))
+    def do_global_search(self, search_id, text):
+        self.queue.append(slskmessages.FileSearch(search_id, text))
 
         """ Request a list of related searches from the server.
         Seemingly non-functional since 2018 (always receiving empty lists). """
 
         # self.queue.append(slskmessages.RelatedSearch(text))
 
-    def do_rooms_search(self, id, text, room=None):
+    def do_rooms_search(self, search_id, text, room=None):
 
         if room is not None:
-            self.queue.append(slskmessages.RoomSearch(room, id, text))
+            self.queue.append(slskmessages.RoomSearch(room, search_id, text))
         else:
-            for room in self.np.chatrooms.joinedrooms:
-                self.queue.append(slskmessages.RoomSearch(room, id, text))
+            for joined_room in self.core.chatrooms.joinedrooms:
+                self.queue.append(slskmessages.RoomSearch(joined_room, search_id, text))
 
-    def do_buddies_search(self, id, text):
+    def do_buddies_search(self, search_id, text):
 
-        for i in self.config.sections["server"]["userlist"]:
-            user = i[0]
-            self.queue.append(slskmessages.UserSearch(user, id, text))
+        for row in self.config.sections["server"]["userlist"]:
+            if row and isinstance(row, list):
+                user = str(row[0])
+                self.queue.append(slskmessages.UserSearch(user, search_id, text))
 
-    def do_peer_search(self, id, text, users):
+    def do_peer_search(self, search_id, text, users):
         for user in users:
-            self.queue.append(slskmessages.UserSearch(user, id, text))
+            self.queue.append(slskmessages.UserSearch(user, search_id, text))
 
-    def do_wishlist_search(self, id, text):
-        self.queue.append(slskmessages.WishlistSearch(id, text))
+    def do_wishlist_search(self, search_id, text):
+        self.add_allowed_search_id(search_id)
+        self.queue.append(slskmessages.WishlistSearch(search_id, text))
 
     def get_current_search_id(self):
         return self.searchid
@@ -143,6 +236,7 @@ class Search:
         if wish not in self.config.sections["server"]["autosearch"]:
             self.config.sections["server"]["autosearch"].append(wish)
 
+        self.add_allowed_search_id(self.searchid)
         return self.searchid
 
     def set_wishlist_interval(self, msg):
@@ -152,13 +246,29 @@ class Search:
 
         log.add_search(_("Wishlist wait period set to %s seconds"), msg.seconds)
 
-    def show_search_result(self, msg, username, country):
-        if self.ui_callback:
-            self.ui_callback.show_search_result(msg, username, country)
+    def file_search_result(self, msg):
+
+        if not self.ui_callback or msg.token not in slskmessages.SEARCH_TOKENS_ALLOWED:
+            return
+
+        conn = msg.conn
+        username = conn.init.target_user
+        addr = conn.addr
+
+        if addr:
+            country = self.geoip.get_country_code(addr[0])
+        else:
+            country = ""
+
+        if country == "-":
+            country = ""
+
+        self.ui_callback.show_search_result(msg, username, country)
 
     """ Incoming search requests """
 
-    def create_search_result_list(self, searchterm, wordindex, maxresults=50):
+    @staticmethod
+    def create_search_result_list(searchterm, wordindex):
 
         try:
             """ Stage 1: Check if each word in the search term is included in our word index.
@@ -166,43 +276,39 @@ class Search:
             word index. If not, exit, since we don't have relevant results. """
 
             largest = 0
+            words = searchterm.split()
 
-            for i in re.finditer(r'\S+', searchterm):
-                i = i.group(0)
+            for word in words:
+                if word not in wordindex:
+                    return None
 
-                if i not in wordindex:
-                    return
-
-                list_size = len(wordindex[i])
+                list_size = len(wordindex[word])
 
                 if list_size > largest:
                     largest = list_size
-                    largest_key = i
+                    largest_key = word
 
             """ Stage 2: Start with the word that has the most file matches, which we selected
             in the previous step, and gradually remove matches that other words in the search
             term don't have. Return the remaining matches, if any. """
 
             results = wordindex[largest_key]
-            searchterm.replace(largest_key, '')
+            words.remove(largest_key)
 
-            for i in re.finditer(r'\S+', searchterm):
-                results = set(results).intersection(wordindex[i.group(0)])
+            for word in words:
+                results = set(results).intersection(wordindex[word])
 
             return results
 
         except ValueError:
             # DB is closed, perhaps when rescanning share or closing Nicotine+
-            return
+            return None
 
     def process_search_request(self, searchterm, user, searchid, direct=False):
         """ Note: since this section is accessed every time a search request arrives,
         several times a second, please keep it as optimized and memory
         sparse as possible! """
 
-        if self.np.transfers is None:
-            return
-
         if not self.config.sections["searches"]["search_results"]:
             # Don't return _any_ results when this option is disabled
             return
@@ -221,14 +327,17 @@ class Search:
             return
 
         # Don't count excluded words as matches (words starting with -)
+        if '-' in searchterm:
+            searchterm = re.sub(r'(\s)-\w+', '', searchterm)
+
         # Strip punctuation
-        searchterm = re.sub(r'(\s)-\w+', '', searchterm).lower().translate(self.translatepunctuation).strip()
+        searchterm = searchterm.lower().translate(self.translatepunctuation).strip()
 
         if len(searchterm) < self.config.sections["searches"]["min_search_chars"]:
             # Don't send search response if search term contains too few characters
             return
 
-        checkuser, reason = self.np.network_filter.check_user(user, None)
+        checkuser, _reason = self.core.network_filter.check_user(user, None)
 
         if not checkuser:
             return
@@ -242,14 +351,15 @@ class Search:
             return
 
         # Find common file matches for each word in search term
-        resultlist = self.create_search_result_list(searchterm, wordindex, maxresults)
+        resultlist = self.create_search_result_list(searchterm, wordindex)
 
         if not resultlist:
             return
 
         numresults = min(len(resultlist), maxresults)
-        queuesize = self.np.transfers.get_upload_queue_size()
-        slotsavail = self.np.transfers.allow_new_uploads()
+        uploadspeed = self.core.transfers.upload_speed
+        queuesize = self.core.transfers.get_upload_queue_size()
+        slotsavail = self.core.transfers.allow_new_uploads()
 
         if checkuser == 2:
             fileindex = self.share_dbs.get("buddyfileindex")
@@ -265,10 +375,10 @@ class Search:
             None,
             self.config.sections["server"]["login"],
             searchid, resultlist, fileindex, slotsavail,
-            self.np.speed, queuesize, fifoqueue, numresults
+            uploadspeed, queuesize, fifoqueue, numresults
         )
 
-        self.np.send_message_to_peer(user, message)
+        self.core.send_message_to_peer(user, message)
 
         if direct:
             log.add_search(
