@@ -20,6 +20,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import gc
 import importlib.util
 import os
 import pickle
@@ -31,7 +32,6 @@ import threading
 import time
 
 from pynicotine import slskmessages
-from pynicotine.i18n import apply_translation
 from pynicotine.logfacility import log
 from pynicotine.utils import rename_process
 
@@ -40,7 +40,7 @@ from pynicotine.utils import rename_process
 if importlib.util.find_spec("_gdbm"):
 
     def shelve_open_gdbm(filename, flag='c', protocol=None, writeback=False):
-        import _gdbm
+        import _gdbm  # pylint: disable=import-error
         return shelve.Shelf(_gdbm.open(filename, flag), protocol, writeback)
 
     shelve.open = shelve_open_gdbm
@@ -48,7 +48,7 @@ if importlib.util.find_spec("_gdbm"):
 elif importlib.util.find_spec("semidbm"):
 
     def shelve_open_semidbm(filename, flag='c', protocol=None, writeback=False):
-        import semidbm
+        import semidbm  # pylint: disable=import-error
         return shelve.Shelf(semidbm.open(filename, flag), protocol, writeback)
 
     shelve.open = shelve_open_semidbm
@@ -67,22 +67,22 @@ class Scanner:
 
     def __init__(self, config, queue, shared_folders, sharestype="normal", rebuild=False):
 
-        from pynicotine.metadata.tinytag import TinyTag
-
         self.config = config
         self.queue = queue
         self.shared_folders = shared_folders
         self.share_dbs = {}
         self.sharestype = sharestype
         self.rebuild = rebuild
-        self.tinytag = TinyTag()
+        self.tinytag = None
         self.translatepunctuation = str.maketrans(dict.fromkeys(string.punctuation, ' '))
 
     def run(self):
 
         try:
+            from pynicotine.metadata.tinytag import TinyTag
+            self.tinytag = TinyTag()
+
             rename_process(b'nicotine-scan')
-            apply_translation()
 
             if self.sharestype == "normal":
                 Shares.load_shares(self.share_dbs, [
@@ -111,14 +111,16 @@ class Scanner:
             from traceback import format_exc
 
             self.queue.put((
-                0, _("Serious error occurred while rescanning shares. If this problem persists, delete %(dir)s/*.db and try again. If that doesn't help, please file a bug report with this stack trace included: %(trace)s"), {
+                _("Serious error occurred while rescanning shares. If this problem persists, "
+                  "delete %(dir)s/*.db and try again. If that doesn't help, please file a bug "
+                  "report with this stack trace included: %(trace)s"), {
                     "dir": self.config.data_dir,
                     "trace": "\n" + format_exc()
-                }
+                }, None
             ))
             self.queue.put(Exception("Scanning failed"))
 
-    def set_shares(self, files=None, streams=None, mtimes=None, wordindex=None, fileindex=None):
+    def set_shares(self, files=None, streams=None, mtimes=None, wordindex=None):
 
         self.config.create_data_folder()
 
@@ -138,12 +140,14 @@ class Scanner:
                     if self.sharestype == "buddy":
                         destination = "buddy" + destination
 
-                    db = shelve.open(os.path.join(self.config.data_dir, destination + ".db"), flag='n', protocol=pickle.HIGHEST_PROTOCOL)
-                    db.update(source)
-                    db.close()
+                    database = shelve.open(os.path.join(self.config.data_dir, destination + ".db"),
+                                           flag='n', protocol=pickle.HIGHEST_PROTOCOL)
+                    database.update(source)
+                    database.close()
 
-                except Exception as e:
-                    self.queue.put((0, _("Can't save %s: %s"), (destination + ".db", e)))
+                except Exception as error:
+                    self.queue.put((_("Can't save %(filename)s: %(error)s"),
+                                    {"filename": destination + ".db", "error": error}, None))
                     return
 
     def rescan_dirs(self, shared, oldmtimes, oldfiles, oldstreams, rebuild=False):
@@ -160,23 +164,22 @@ class Scanner:
         except TypeError:
             num_folders = len(list(oldmtimes))
 
-        self.queue.put((0, _("%(num)s folders found before rescan, rebuilding..."), {"num": num_folders}))
+        self.queue.put((_("%(num)s folders found before rescan, rebuilding..."), {"num": num_folders}, None))
 
         newmtimes = {}
 
         for folder in shared_directories:
-            if not self.is_hidden(folder):
-                # Get mtimes for top-level shared folders, then every subfolder
-                try:
-                    mtime = os.stat(folder).st_mtime
-                    newmtimes[folder] = mtime
-                    newmtimes = {**newmtimes, **self.get_folder_mtimes(folder)}
+            # Get mtimes for top-level shared folders, then every subfolder
+            try:
+                mtime = os.stat(folder).st_mtime
+                newmtimes[folder] = mtime
+                newmtimes = {**newmtimes, **self.get_folder_mtimes(folder)}
 
-                except OSError as errtuple:
-                    self.queue.put((0, _("Error while scanning folder %(path)s: %(error)s"), {
-                        'path': folder,
-                        'error': errtuple
-                    }))
+            except OSError as errtuple:
+                self.queue.put((_("Error while scanning folder %(path)s: %(error)s"), {
+                    'path': folder,
+                    'error': errtuple
+                }, None))
 
         # Get list of files
         # returns dict in format { Directory : { File : metadata, ... }, ... }
@@ -185,6 +188,9 @@ class Scanner:
 
         # Save data to databases
         self.set_shares(files=newsharedfiles, streams=newsharedfilesstreams, mtimes=newmtimes)
+        del newsharedfilesstreams
+        del newmtimes
+        gc.collect()
 
         # Update Search Index
         # wordindex is a dict in format {word: [num, num, ..], ... } with num matching keys in newfileindex
@@ -193,19 +199,21 @@ class Scanner:
 
         # Save data to databases
         self.set_shares(wordindex=wordindex)
+        self.queue.put((_("%(num)s folders found after rescan"), {"num": len(newsharedfiles)}, None))
+        del wordindex
+        del newsharedfiles
+        gc.collect()
 
-        self.queue.put((0, _("%(num)s folders found after rescan"), {"num": len(newsharedfiles)}))
-
-    def is_hidden(self, folder, filename=None, folder_obj=None):
+    @staticmethod
+    def is_hidden(folder, filename=None, folder_obj=None):
         """ Stop sharing any dot/hidden directories/files """
 
-        # If any part of the directory structure start with a dot we exclude it
+        # If the last folder in the path starts with a dot, we exclude it
         if filename is None:
-            subfolders = folder.replace('\\', '/').split('/')
+            last_folder = os.path.basename(os.path.normpath(folder.replace('\\', '/')))
 
-            for part in subfolders:
-                if part.startswith("."):
-                    return True
+            if last_folder.startswith("."):
+                return True
 
         # If we're asked to check a file we exclude it if it start with a dot
         if filename is not None and filename.startswith("."):
@@ -228,7 +236,8 @@ class Scanner:
 
         return False
 
-    def get_utf8_path(self, path):
+    @staticmethod
+    def get_utf8_path(path):
         """ Convert a path to utf-8, if necessary """
 
         try:
@@ -256,20 +265,21 @@ class Scanner:
                         mtime = entry.stat().st_mtime
 
                     except OSError as errtuple:
-                        self.queue.put((0, _("Error while scanning %(path)s: %(error)s"), {
+                        self.queue.put((_("Error while scanning %(path)s: %(error)s"), {
                             'path': path,
                             'error': errtuple
-                        }))
+                        }, None))
                         continue
 
                     mtimes[path] = mtime
                     dircontents = self.get_folder_mtimes(path)
 
-                    for k in dircontents:
-                        mtimes[k] = dircontents[k]
+                    for k, contents in dircontents.items():
+                        mtimes[k] = contents
 
         except OSError as errtuple:
-            self.queue.put((0, _("Error while scanning folder %(path)s: %(error)s"), {'path': folder, 'error': errtuple}))
+            self.queue.put((_("Error while scanning folder %(path)s: %(error)s"),
+                           {'path': folder, 'error': errtuple}, None))
 
         return mtimes
 
@@ -289,27 +299,26 @@ class Scanner:
                 # Truncate the percentage to two decimal places to avoid sending data to the GUI thread too often
                 percent = float("%.2f" % (float(count) / len(mtimes) * 0.5))
 
-                if percent > lastpercent and percent <= 1.0:
+                if lastpercent < percent <= 1.0:
                     self.queue.put(percent)
                     lastpercent = percent
 
-                virtualdir = Shares._real2virtual(folder, self.config)
+                virtualdir = Shares.real2virtual_cls(folder, self.config)
 
-                if not rebuild and folder in oldmtimes:
-                    if mtimes[folder] == oldmtimes[folder]:
-                        if os.path.exists(folder):
-                            try:
-                                files[virtualdir] = oldfiles[virtualdir]
-                                streams[virtualdir] = oldstreams[virtualdir]
-                                continue
-                            except KeyError:
-                                self.queue.put((6, _("Inconsistent cache for '%(vdir)s', rebuilding '%(dir)s'"), {
-                                    'vdir': virtualdir,
-                                    'dir': folder
-                                }))
-                        else:
-                            self.queue.put((6, _("Dropping missing folder %(dir)s"), {'dir': folder}))
+                if not rebuild and folder in oldmtimes and mtimes[folder] == oldmtimes[folder]:
+                    if os.path.exists(folder):
+                        try:
+                            files[virtualdir] = oldfiles[virtualdir]
+                            streams[virtualdir] = oldstreams[virtualdir]
                             continue
+                        except KeyError:
+                            self.queue.put((_("Inconsistent cache for '%(vdir)s', rebuilding '%(dir)s'"), {
+                                'vdir': virtualdir,
+                                'dir': folder
+                            }, "debug"))
+                    else:
+                        self.queue.put((_("Dropping missing folder %(dir)s"), {'dir': folder}, "debug"))
+                        continue
 
                 files[virtualdir] = []
 
@@ -322,27 +331,29 @@ class Scanner:
                             continue
 
                         # Get the metadata of the file
-                        data = self.get_file_info(filename, entry.path, self.tinytag, self.queue, entry)
-                        if data is not None:
-                            files[virtualdir].append(data)
+                        path = self.get_utf8_path(entry.path)
+                        data = self.get_file_info(filename, path, self.tinytag, self.queue, entry)
+                        files[virtualdir].append(data)
 
                 streams[virtualdir] = self.get_dir_stream(files[virtualdir])
 
             except OSError as errtuple:
-                self.queue.put((0, _("Error while scanning folder %(path)s: %(error)s"), {'path': folder, 'error': errtuple}))
+                self.queue.put((_("Error while scanning folder %(path)s: %(error)s"),
+                               {'path': folder, 'error': errtuple}, None))
                 continue
 
         return files, streams
 
-    @classmethod
-    def get_file_info(cls, name, pathname, tinytag, queue=None, file=None):
+    @staticmethod
+    def get_file_info(name, pathname, tinytag, queue=None, file=None):
         """ Get file metadata """
 
-        try:
-            audio = None
-            bitrateinfo = None
-            duration = None
+        size = 0
+        audio = None
+        bitrateinfo = None
+        duration = None
 
+        try:
             if file:
                 # Faster way if we use scandir
                 size = file.stat().st_size
@@ -362,7 +373,7 @@ class Scanner:
                     }
 
                     if queue:
-                        queue.put((0, error, args))
+                        queue.put((error, args, None))
                     else:
                         log.add(error, args)
 
@@ -373,8 +384,6 @@ class Scanner:
                 if audio.duration is not None:
                     duration = int(audio.duration)
 
-            return (name, size, bitrateinfo, duration)
-
         except Exception as errtuple:
             error = _("Error while scanning file %(path)s: %(error)s")
             args = {
@@ -383,17 +392,19 @@ class Scanner:
             }
 
             if queue:
-                queue.put((0, error, args))
+                queue.put((error, args, None))
             else:
                 log.add(error, args)
 
-    @classmethod
-    def get_dir_stream(cls, folder):
+        return (name, size, bitrateinfo, duration)
+
+    @staticmethod
+    def get_dir_stream(folder):
         """ Pack all files and metadata in directory """
 
         message = slskmessages.SlskMessage()
         stream = bytearray()
-        stream.extend(message.pack_object(len(folder), unsignedint=True))
+        stream.extend(message.pack_object(len(folder)))
 
         for fileinfo in folder:
             stream.extend(bytes([1]))
@@ -406,7 +417,7 @@ class Scanner:
 
                 stream.extend(message.pack_object(0))
                 try:
-                    stream.extend(message.pack_object(fileinfo[2][0], unsignedint=True))
+                    stream.extend(message.pack_object(fileinfo[2][0]))
 
                 except Exception:
                     # Invalid bitrate
@@ -414,7 +425,7 @@ class Scanner:
 
                 stream.extend(message.pack_object(1))
                 try:
-                    stream.extend(message.pack_object(fileinfo[3], unsignedint=True))
+                    stream.extend(message.pack_object(fileinfo[3]))
 
                 except Exception:
                     # Invalid length
@@ -434,8 +445,8 @@ class Scanner:
 
         return stream
 
-    @classmethod
-    def add_file_to_index(cls, index, filename, folder, fileinfo, wordindex, fileindex, pattern):
+    @staticmethod
+    def add_file_to_index(index, filename, folder, fileinfo, wordindex, fileindex, pattern):
         """ Add a file to the file index database """
 
         fileindex[repr(index)] = (folder + '\\' + filename, *fileinfo[1:])
@@ -460,7 +471,8 @@ class Scanner:
         else:
             fileindex_dest = "buddyfileindex"
 
-        fileindex = shelve.open(os.path.join(self.config.data_dir, fileindex_dest + ".db"), flag='n', protocol=pickle.HIGHEST_PROTOCOL)
+        fileindex = shelve.open(os.path.join(self.config.data_dir, fileindex_dest + ".db"),
+                                flag='n', protocol=pickle.HIGHEST_PROTOCOL)
         wordindex = {}
 
         index = 0
@@ -473,12 +485,13 @@ class Scanner:
             # Truncate the percentage to two decimal places to avoid sending data to the GUI thread too often
             percent = float("%.2f" % (float(count) / len(sharedfiles) * 0.5))
 
-            if percent > lastpercent and percent <= 1.0:
+            if lastpercent < percent <= 1.0:
                 self.queue.put(percent)
                 lastpercent = percent
 
             for fileinfo in sharedfiles[folder]:
-                self.add_file_to_index(index, fileinfo[0], folder, fileinfo, wordindex, fileindex, self.translatepunctuation)
+                self.add_file_to_index(
+                    index, fileinfo[0], folder, fileinfo, wordindex, fileindex, self.translatepunctuation)
                 index += 1
 
         fileindex.close()
@@ -487,9 +500,9 @@ class Scanner:
 
 class Shares:
 
-    def __init__(self, np, config, queue, ui_callback=None, connected=False):
+    def __init__(self, core, config, queue, ui_callback=None, connected=False):
 
-        self.np = np
+        self.core = core
         self.ui_callback = ui_callback
         self.config = config
         self.queue = queue
@@ -534,14 +547,14 @@ class Shares:
     """ Shares-related actions """
 
     def real2virtual(self, path):
-        return self._real2virtual(path, self.config)
+        return self.real2virtual_cls(path, self.config)
 
     @classmethod
-    def _real2virtual(cls, path, config):
+    def real2virtual_cls(cls, path, config):
 
         path = path.replace('/', '\\')
 
-        for (virtual, real, *unused) in cls._virtualmapping(config):
+        for (virtual, real, *_unused) in cls._virtualmapping(config):
             # Remove slashes from share name to avoid path conflicts
             virtual = virtual.replace('/', '_').replace('\\', '_')
 
@@ -562,7 +575,7 @@ class Shares:
 
         path = path.replace('/', os.sep).replace('\\', os.sep)
 
-        for (virtual, real, *unused) in self._virtualmapping(self.config):
+        for (virtual, real, *_unused) in self._virtualmapping(self.config):
             # Remove slashes from share name to avoid path conflicts
             virtual = virtual.replace('/', '_').replace('\\', '_')
 
@@ -575,8 +588,8 @@ class Shares:
 
         return "__INTERNAL_ERROR__" + path
 
-    @classmethod
-    def _virtualmapping(cls, config):
+    @staticmethod
+    def _virtualmapping(config):
 
         mapping = config.sections["transfers"]["shared"][:]
 
@@ -590,7 +603,8 @@ class Shares:
 
     def init_shares(self):
 
-        rescan_startup = self.config.sections["transfers"]["rescanonstartup"]
+        rescan_startup = (self.config.sections["transfers"]["rescanonstartup"]
+                          and not self.config.need_config())
 
         # Rescan public shares if necessary
         if not self.config.sections["transfers"]["friendsonly"]:
@@ -613,15 +627,19 @@ class Shares:
     def convert_shares(self):
         """ Convert fs-based shared to virtual shared (pre 1.4.0) """
 
-        def _convert_to_virtual(x):
-            if isinstance(x, tuple):
-                return x
-            virtual = x.replace('/', '_').replace('\\', '_').strip('_')
-            log.add("Renaming shared folder '%s' to '%s'. A rescan of your share is required.", (x, virtual))
-            return (virtual, x)
+        def _convert_to_virtual(shared_folder):
+            if isinstance(shared_folder, tuple):
+                return shared_folder
 
-        self.config.sections["transfers"]["shared"] = [_convert_to_virtual(x) for x in self.config.sections["transfers"]["shared"]]
-        self.config.sections["transfers"]["buddyshared"] = [_convert_to_virtual(x) for x in self.config.sections["transfers"]["buddyshared"]]
+            virtual = shared_folder.replace('/', '_').replace('\\', '_').strip('_')
+            log.add("Renaming shared folder '%s' to '%s'. A rescan of your share is required.",
+                    (shared_folder, virtual))
+            return (virtual, shared_folder)
+
+        self.config.sections["transfers"]["shared"] = [_convert_to_virtual(x)
+                                                       for x in self.config.sections["transfers"]["shared"]]
+        self.config.sections["transfers"]["buddyshared"] = [_convert_to_virtual(x)
+                                                            for x in self.config.sections["transfers"]["buddyshared"]]
 
     @classmethod
     def load_shares(cls, shares, dbs, reset_shares=False):
@@ -643,11 +661,11 @@ class Shares:
                 exception = format_exc()
 
         if errors:
-            log.add_warning(_("Failed to process the following databases: %(names)s"), {
+            log.add(_("Failed to process the following databases: %(names)s"), {
                 'names': '\n'.join(errors)
             })
-            log.add_warning(exception)
-            log.add_warning(_("Shared files database seems to be corrupted, rescan your shares"))
+            log.add(exception)
+            log.add(_("Shared files database seems to be corrupted, rescan your shares"))
 
             if not reset_shares:
                 cls.load_shares(shares, dbs, reset_shares=True)
@@ -666,12 +684,19 @@ class Shares:
             })
             return False
 
-        folder, sep, file = virtualfilename.rpartition('\\')
+        folder, _sep, file = virtualfilename.rpartition('\\')
         shared = self.share_dbs.get("files")
         bshared = self.share_dbs.get("buddyfiles")
 
         if self.config.sections["transfers"]["enablebuddyshares"] and bshared is not None:
-            if user in (i[0] for i in self.config.sections["server"]["userlist"]):
+            for row in self.config.sections["server"]["userlist"]:
+                if row[0] != user:
+                    continue
+
+                # Check if buddy is trusted
+                if self.config.sections["transfers"]["buddysharestrustedonly"] and not row[4]:
+                    break
+
                 for i in bshared.get(str(folder), ''):
                     if file == i[0]:
                         return True
@@ -722,7 +747,8 @@ class Shares:
             except TypeError:
                 index = len(list(fileindex))
 
-            Scanner.add_file_to_index(index, file, vdir, fileinfo, self.share_dbs["wordindex"], fileindex, self.translatepunctuation)
+            Scanner.add_file_to_index(
+                index, file, vdir, fileinfo, self.share_dbs["wordindex"], fileindex, self.translatepunctuation)
 
             self.share_dbs["mtimes"][vdir] = os.path.getmtime(rdir)
             self.newnormalshares = True
@@ -793,8 +819,8 @@ class Shares:
         """ Returns the compressed shares message. Creates a new one if necessary, e.g.
         if an individual file was added to our shares. """
 
-        if sharestype == "normal" and self.newnormalshares or \
-                sharestype == "buddy" and self.newbuddyshares:
+        if (sharestype == "normal" and self.newnormalshares
+                or sharestype == "buddy" and self.newbuddyshares):
             self.create_compressed_shares_message(sharestype)
 
         if sharestype == "normal":
@@ -805,15 +831,19 @@ class Shares:
             self.newbuddyshares = False
             return self.compressed_shares_buddy
 
+        return None
+
     def compress_shares(self, sharestype):
         """ Begin compressing the shares list. This compressed list will be used to
         quickly send our file list to users. """
 
         if sharestype == "normal":
-            thread = threading.Thread(target=self.compressed_shares_normal.make_network_message, args=(0, True))
+            self.compressed_shares_normal.built = None
+            thread = threading.Thread(target=self.compressed_shares_normal.make_network_message)
 
         elif sharestype == "buddy":
-            thread = threading.Thread(target=self.compressed_shares_buddy.make_network_message, args=(0, True))
+            self.compressed_shares_buddy.built = None
+            thread = threading.Thread(target=self.compressed_shares_buddy.make_network_message)
 
         thread.name = "CompressShares"
         thread.daemon = True
@@ -832,12 +862,12 @@ class Shares:
                 "buddyfileindex", "buddymtimes"
             ]
 
-        for db in dbs:
-            db_file = self.share_dbs.get(db)
+        for database in dbs:
+            db_file = self.share_dbs.get(database)
 
             if db_file is not None:
-                self.share_dbs[db].close()
-                del self.share_dbs[db]
+                self.share_dbs[database].close()
+                del self.share_dbs[database]
 
     def send_num_shared_folders_files(self):
         """ Send number publicly shared files to the server. """
@@ -845,7 +875,7 @@ class Shares:
         if self.config.sections["transfers"]["friendsonly"]:
             # No public shares
             files = folders = 0
-            self.queue.append(slskmessages.SharedFoldersFiles(files, folders))
+            self.queue.append(slskmessages.SharedFoldersFiles(folders, files))
             return
 
         try:
@@ -865,8 +895,8 @@ class Shares:
 
             self.queue.append(slskmessages.SharedFoldersFiles(sharedfolders, sharedfiles))
 
-        except Exception as e:
-            log.add(_("Failed to send number of shared files to the server: %s"), e)
+        except Exception as error:
+            log.add(_("Failed to send number of shared files to the server: %s"), error)
 
     """ Scanning """
 
@@ -889,16 +919,16 @@ class Shares:
         return scanner, scanner_queue
 
     def rebuild_public_shares(self, thread=True):
-        self.rescan_shares("normal", rebuild=True, use_thread=thread)
+        return self.rescan_shares("normal", rebuild=True, use_thread=thread)
 
     def rescan_public_shares(self, rebuild=False, thread=True):
-        self.rescan_shares("normal", rebuild, thread)
+        return self.rescan_shares("normal", rebuild, thread)
 
     def rebuild_buddy_shares(self, thread=True):
-        self.rescan_shares("buddy", rebuild=True, use_thread=thread)
+        return self.rescan_shares("buddy", rebuild=True, use_thread=thread)
 
     def rescan_buddy_shares(self, rebuild=False, thread=True):
-        self.rescan_shares("buddy", rebuild, thread)
+        return self.rescan_shares("buddy", rebuild, thread)
 
     def get_shared_folders(self, sharestype):
 
@@ -906,7 +936,8 @@ class Shares:
             shared_folders = self.config.sections["transfers"]["shared"][:]
 
         else:
-            shared_folders = self.config.sections["transfers"]["buddyshared"][:] + self.config.sections["transfers"]["shared"][:]
+            shared_folders = (self.config.sections["transfers"]["buddyshared"][:]
+                              + self.config.sections["transfers"]["shared"][:])
 
         if self.config.sections["transfers"]["sharedownloaddir"]:
             shared_folders.append((_('Downloaded'), self.config.sections["transfers"]["downloaddir"]))
@@ -925,8 +956,8 @@ class Shares:
                 if isinstance(item, Exception):
                     return True
 
-                elif isinstance(item, tuple):
-                    log_level, template, args = item
+                if isinstance(item, tuple):
+                    template, args, log_level = item
                     log.add(template, args, log_level)
 
                 elif isinstance(item, float):
@@ -940,10 +971,10 @@ class Shares:
     def rescan_shares(self, sharestype, rebuild=False, use_thread=True):
 
         if sharestype == "normal" and self.public_rescanning:
-            return
+            return None
 
         if sharestype == "buddy" and self.buddy_rescanning:
-            return
+            return None
 
         if sharestype == "normal":
             log.add(_("Rescanning normal shares..."))
@@ -966,8 +997,9 @@ class Shares:
             thread.name = "ProcessShareScanner"
             thread.daemon = True
             thread.start()
-        else:
-            self._process_scanner(scanner, scanner_queue, sharestype)
+            return None
+
+        return self._process_scanner(scanner, scanner_queue, sharestype)
 
     def _process_scanner(self, scanner, scanner_queue, sharestype):
 
@@ -1006,3 +1038,5 @@ class Shares:
 
         if self.ui_callback:
             self.ui_callback.hide_scan_progress(sharestype)
+
+        return error
