@@ -28,21 +28,16 @@ This module contains utility functions.
 import errno
 import os
 import pickle
-import sys
 
 from pynicotine.config import config
 from pynicotine.logfacility import log
 
-version = "3.0.6"
-
-win32 = sys.platform.startswith("win")
-
-illegalpathchars = ["?", ":", ">", "<", "|", "*", '"']
-illegalfilechars = illegalpathchars + ["\\", "/"]
-replacementchar = '_'
+ILLEGALPATHCHARS = ["?", ":", ">", "<", "|", "*", '"']
+ILLEGALFILECHARS = ILLEGALPATHCHARS + ["\\", "/"]
+REPLACEMENTCHAR = '_'
 
 
-def rename_process(new_name, debug=False):
+def rename_process(new_name, debug_info=False):
 
     errors = []
 
@@ -53,8 +48,8 @@ def rename_process(new_name, debug=False):
         libc = ctypes.CDLL(None)
         libc.prctl(15, new_name, 0, 0, 0)
 
-    except Exception as e:
-        errors.append(e)
+    except Exception as error:
+        errors.append(error)
         errors.append("Failed GNU/Linux style")
 
         try:
@@ -63,11 +58,11 @@ def rename_process(new_name, debug=False):
             libc = ctypes.CDLL(None)
             libc.setproctitle(new_name)
 
-        except Exception as e:
-            errors.append(e)
+        except Exception as error:
+            errors.append(error)
             errors.append("Failed BSD style")
 
-    if debug and errors:
+    if debug_info and errors:
         msg = ["Errors occurred while trying to change process name:"]
         for i in errors:
             msg.append("%s" % (i,))
@@ -76,8 +71,8 @@ def rename_process(new_name, debug=False):
 
 def clean_file(filename):
 
-    for char in illegalfilechars:
-        filename = filename.replace(char, replacementchar)
+    for char in ILLEGALFILECHARS:
+        filename = filename.replace(char, REPLACEMENTCHAR)
 
     return filename
 
@@ -92,8 +87,8 @@ def clean_path(path, absolute=False):
         drive = path[:3]
         path = path[3:]
 
-    for char in illegalpathchars:
-        path = path.replace(char, replacementchar)
+    for char in ILLEGALPATHCHARS:
+        path = path.replace(char, REPLACEMENTCHAR)
 
     path = ''.join([drive, path])
 
@@ -111,10 +106,10 @@ def get_path(folder_name, base_name, callback, data=None):
         filepath = os.path.join(folder_name, base_name)
         callback(filepath, data)
 
-    except OSError as e:
-        if e.errno != errno.EINVAL:
+    except OSError as error:
+        if error.errno != errno.EINVAL:
             # The issue is not caused by invalid path characters, raise error as usual
-            raise OSError(e)
+            raise OSError from error
 
         # Use path with forbidden characters removed (NTFS/FAT)
         filepath = os.path.join(folder_name, clean_file(base_name))
@@ -144,12 +139,10 @@ def get_latest_version():
 
 def make_version(version):
 
-    s = version.split(".")
-    major, minor, patch = (int(i) for i in s[:3])
+    major, minor, patch = (int(i) for i in version.split(".")[:3])
     stable = 1
 
-    if "dev" in version or \
-            "rc" in version:
+    if "dev" in version or "rc" in version:
         # Example: 2.0.1.dev1
         # A dev version will be one less than a stable version
         stable = 0
@@ -175,7 +168,7 @@ def get_result_bitrate_length(filesize, attributes):
         third = attributes[2]
 
         # Sometimes the vbr indicator is in third position
-        if third == 0 or third == 1:
+        if third in (0, 1):
 
             if third == 1:
                 h_bitrate = " (vbr)"
@@ -187,7 +180,7 @@ def get_result_bitrate_length(filesize, attributes):
             h_length = '%i:%02i' % (second / 60, second % 60)
 
         # Sometimes the vbr indicator is in second position
-        elif second == 0 or second == 1:
+        elif second in (0, 1):
 
             if second == 1:
                 h_bitrate = " (vbr)"
@@ -221,7 +214,7 @@ def get_result_bitrate_length(filesize, attributes):
         second = attributes[1]
 
         # Sometimes the vbr indicator is in second position
-        if second == 0 or second == 1:
+        if second in (0, 1):
 
             # If it's a vbr file we can't deduce the length
             if second == 1:
@@ -269,8 +262,11 @@ def human_size(filesize):
                 return "%3.1f %s" % (filesize, i)
 
             filesize /= step_unit
+
     except TypeError:
-        return filesize
+        pass
+
+    return filesize
 
 
 speed_suffixes = ['B/s', 'KiB/s', 'MiB/s', 'GiB/s', 'TiB/s', 'PiB/s', 'EiB/s', 'ZiB/s', 'YiB/s']
@@ -285,18 +281,21 @@ def human_speed(filesize):
                 return "%3.1f %s" % (filesize, i)
 
             filesize /= step_unit
+
     except TypeError:
-        return filesize
+        pass
+
+    return filesize
 
 
 def humanize(number):
 
     fashion = config.sections["ui"]["decimalsep"]
 
-    if fashion == "" or fashion == "<None>":
+    if fashion in ("", "<None>"):
         return str(number)
 
-    elif fashion == "<space>":
+    if fashion == "<space>":
         fashion = " "
 
     number = str(number)
@@ -367,7 +366,8 @@ def execute_command(command, replacement=None, background=True, returnoutput=Fal
     if command.endswith("&"):
         command = command[:-1]
         if returnoutput:
-            print("Yikes, I was asked to return output but I'm also asked to launch the process in the background. returnoutput gets precedent.")
+            print("Yikes, I was asked to return output but I'm also asked to launch "
+                  "the process in the background. returnoutput gets precedent.")
         else:
             background = True
 
@@ -401,7 +401,7 @@ def execute_command(command, replacement=None, background=True, returnoutput=Fal
 
     # subcommands is now: [['C:\Program Files\WinAmp\WinAmp.exe', '--xforce', '--title=My Title', '$'], ['flite', '-t']]
     if replacement:
-        for i in range(0, len(subcommands)):
+        for i, _ in enumerate(subcommands):
             subcommands[i] = [x.replace(placeholder, replacement) for x in subcommands[i]]
 
     # Chaining commands...
@@ -421,8 +421,9 @@ def execute_command(command, replacement=None, background=True, returnoutput=Fal
             procs.append(Popen(subcommands[-1], stdin=procs[-1].stdout, stdout=finalstdout))
         if not background and not returnoutput:
             procs[-1].wait()
-    except Exception:
-        raise RuntimeError("Problem while executing command %s (%s of %s)" % (subcommands[len(procs)], len(procs) + 1, len(subcommands)))
+    except Exception as error:
+        raise RuntimeError("Problem while executing command %s (%s of %s)" %
+                           (subcommands[len(procs)], len(procs) + 1, len(subcommands))) from error
 
     if not returnoutput:
         return True
@@ -430,6 +431,34 @@ def execute_command(command, replacement=None, background=True, returnoutput=Fal
     return procs[-1].communicate()[0]
 
 
+def load_file(path, load_func, use_old_file=False):
+
+    try:
+        if use_old_file:
+            path = path + ".old"
+
+        elif os.path.isfile(path + ".old"):
+            if not os.path.isfile(path):
+                raise OSError("*.old file is present but main file is missing")
+
+            if os.path.getsize(path) == 0:
+                # Empty files should be considered broken/corrupted
+                raise OSError("*.old file is present but main file is empty")
+
+        return load_func(path)
+
+    except Exception as error:
+        log.add(_("Something went wrong while reading file %(filename)s: %(error)s"),
+                {"filename": path, "error": error})
+
+        if not use_old_file:
+            # Attempt to load data from .old file
+            log.add(_("Attempting to load backup of file %s"), path)
+            return load_file(path, load_func, use_old_file=True)
+
+    return None
+
+
 def write_file_and_backup(path, callback, protect=False):
 
     # Back up old file to path.old
@@ -442,7 +471,7 @@ def write_file_and_backup(path, callback, protect=False):
                 os.chmod(path + ".old", 0o600)
 
     except Exception as error:
-        log.add_warning(_("Unable to back up file %(path)s: %(error)s"), {
+        log.add(_("Unable to back up file %(path)s: %(error)s"), {
             "path": path,
             "error": error
         })
@@ -452,11 +481,11 @@ def write_file_and_backup(path, callback, protect=False):
         oldumask = os.umask(0o077)
 
     try:
-        with open(path, "w", encoding="utf-8") as f:
-            callback(f)
+        with open(path, "w", encoding="utf-8") as file_handle:
+            callback(file_handle)
 
     except Exception as error:
-        log.add_warning(_("Unable to save file %(path)s: %(error)s"), {
+        log.add(_("Unable to save file %(path)s: %(error)s"), {
             "path": path,
             "error": error
         })
@@ -467,7 +496,7 @@ def write_file_and_backup(path, callback, protect=False):
                 os.rename(path + ".old", path)
 
         except Exception as error:
-            log.add_warning(_("Unable to restore previous file %(path)s: %(error)s"), {
+            log.add(_("Unable to restore previous file %(path)s: %(error)s"), {
                 "path": path,
                 "error": error
             })
@@ -476,13 +505,16 @@ def write_file_and_backup(path, callback, protect=False):
         os.umask(oldumask)
 
 
-def http_request(url_scheme, base_url, path, request_type="GET", body="", headers={}, timeout=10, redirect_depth=0):
+def http_request(url_scheme, base_url, path, request_type="GET", body="", headers=None, timeout=10, redirect_depth=0):
 
-    if redirect_depth > 15:
-        raise Exception("Redirected too many times, giving up")
+    if headers is None:
+        headers = {}
 
     import http.client
 
+    if redirect_depth > 15:
+        raise http.client.HTTPException("Redirected too many times, giving up")
+
     if url_scheme == "https":
         conn = http.client.HTTPSConnection(base_url, timeout=timeout)
     else:
@@ -537,16 +569,15 @@ def add_alias(rest):
 
         if args[0] in aliases:
             return "Alias %s: %s\n" % (args[0], aliases[args[0]])
-        else:
-            return _("No such alias (%s)") % rest + "\n"
 
-    else:
-        m = "\n" + _("Aliases:") + "\n"
+        return _("No such alias (%s)") % rest + "\n"
 
-        for (key, value) in aliases.items():
-            m = m + "%s: %s\n" % (key, value)
+    msg = "\n" + _("Aliases:") + "\n"
 
-        return m + "\n"
+    for (key, value) in aliases.items():
+        msg = msg + "%s: %s\n" % (key, value)
+
+    return msg + "\n"
 
 
 def unalias(rest):
@@ -554,13 +585,12 @@ def unalias(rest):
     aliases = config.sections["server"]["command_aliases"]
 
     if rest and rest in aliases:
-        x = aliases[rest]
+        action = aliases[rest]
         del aliases[rest]
 
-        return _("Removed alias %(alias)s: %(action)s\n") % {'alias': rest, 'action': x}
+        return _("Removed alias %(alias)s: %(action)s\n") % {'alias': rest, 'action': action}
 
-    else:
-        return _("No such alias (%(alias)s)\n") % {'alias': rest}
+    return _("No such alias (%(alias)s)\n") % {'alias': rest}
 
 
 def is_alias(cmd):
@@ -589,19 +619,18 @@ def _expand_alias(aliases, cmd):
     def getpart(line):
         if line[0] != "(":
             return ""
-        ix = 1
+        i = 1
         ret = ""
         level = 0
-        while ix < len(line):
-            if line[ix] == "(":
+        while i < len(line):
+            if line[i] == "(":
                 level = level + 1
-            if line[ix] == ")":
+            if line[i] == ")":
                 if level == 0:
                     return ret
-                else:
-                    level = level - 1
-            ret = ret + line[ix]
-            ix = ix + 1
+                level = level - 1
+            ret = ret + line[i]
+            i = i + 1
         return ""
 
     if not is_alias(cmd):
@@ -636,16 +665,16 @@ def _expand_alias(aliases, cmd):
                         last = int(args[1])
                     else:
                         last = len(cmd)
-                v = " ".join(cmd[first:last + 1])
-                if not v:
-                    v = default
-                ret = ret + v
+                value = " ".join(cmd[first:last + 1])
+                if not value:
+                    value = default
+                ret = ret + value
             else:
                 ret = ret + alias[i]
                 i = i + 1
         return ret
     except Exception as error:
-        log.add_warning("%s", error)
+        log.add("%s", error)
 
     return ""
 
@@ -667,14 +696,13 @@ def get_completion_list(commands, rooms):
 
     if config_words["buddies"]:
         for i in config.sections["server"]["userlist"]:
-            user = i[0]
-            if isinstance(user, str):
+            if i and isinstance(i, list):
+                user = str(i[0])
                 completion_list.append(user)
 
     if config_words["aliases"]:
         for k in config.sections["server"]["command_aliases"].keys():
-            if isinstance(k, str):
-                completion_list.append("/" + k)
+            completion_list.append("/" + str(k))
 
     if config_words["commands"]:
         completion_list += commands
